#!/bin/bash
####################################################################
# (C) Copyright 2017 Hewlett Packard Enterprise Development L.P.
# @(#) HPE Serviceguard Oracle ASM MIRROR Configuration Script
# @(#) Product Name                :  serviceguard-oracle-toolkit
# @(#) Product Version             :  A.12.20.00
# @(#) Patch Name                  :  
#
# *** Note: This file MUST NOT be edited. *****
#
# Any changes made to it will be overwritten when you upgrade to the
# next release of HPE Serviceguard Oracle Toolkit
#
# Changing this file may lead to unrecoverable Serviceguard Oracle
# toolkit package startup and/or shutdown problems.
#
###################################################################
#
# This script can be invoked from master_control_script.
#########################################
. /etc/cmcluster.conf

#########################################
# Source ASM MIRROR utility functions.
##########################################
ASM_MIRROR_UTILS=${SGCONF}/scripts/tkit/oracle/asm_mirror_utils.sh
export ASM_MIRROR_UTILS

# Set PATH to reference the appropriate directories.
PATH=$SGSBIN:/bin:/sbin:/usr/bin:/usr/sbin

if [[ -f ${ASM_MIRROR_UTILS} ]]; then
    . ${ASM_MIRROR_UTILS}
    if [[ $? -ne 0 ]]
    then
        echo "ERROR: Unable to source package Oracle ASM mirror "\
             "utility functions file: ${ASM_MIRROR_UTILS}"
        exit 1
    fi
else
    echo "ERROR: Unable to find Oracle ASM mirror utility functions "\
         "file: ${ASM_MIRROR_UTILS}"
    exit 1
fi

###################################################################
#
# Get the environment for this package through utility function
# sg_source_pkg_env().
#
###################################################################

sg_source_pkg_env $*

# Global variable
# To find if any new ASM_PV has been added or removed from package. 
typeset -i CHANGE_IN_ASM_PV_LIST=$FALSE

#######################################################################
#
# function ora_start_asm_mirror() is called by master control script
# during Oracle package startup with ASM mirror.
#
#######################################################################

function ora_start_asm_mirror
{
    typeset pkg=${SG_PACKAGE_NAME}
    typeset dg_index=0
    sg_log 5 "Function ora_start_asm_mirror"

    # Remove old backuped ASM Mirror conf or disk group status files
    # if exists.
    cleanup_backup_conf_status_file

    # Caching all diskgroup, failgroup related info and sync status of 
    # all diskgroup. 
    cache_dg_info
    if [[ $? -ne 0 ]];then 
        to_exit=1
        # Below will set global flag so that all other 
        # package module will get to know there is 
        # a failure while reconfiguration (while adding 
        # new module online).
        if sg_check_if_it_is_reconfiguration
        then
            sg_set_reconfig_failure
        fi
        return
    fi
    # Populate all asm device related info like asm disk name, diskgroup
    # and failgroup. 
    populate_asm_dev_list_info "start"
    if [[ $? -ne 0 ]];then
        to_exit=1
        # Below will set global flag so that all other 
        # package module will get to know there is 
        # a failure while reconfiguration (while adding 
        # new module online).
        if sg_check_if_it_is_reconfiguration
        then
            sg_set_reconfig_failure
        fi
        return
    fi
    sg_log 0 "Checking for ASM mirroring status of all configured"\
             "Oracle ASM diskgroups against RPO criteria"\
             "on \"$HOST\" ..."
    if [[ "${ORA_ASM_MIRROR_RPO_TARGET}" == "0" ]];then
        while (( $dg_index < ${#IS_FG_SYNC_IN_DG[@]} ))
            do
            # Below will check if reconfiguration has already been failed 
            # while adding new module online to a package. 
            if sg_check_if_reconfig_has_failed
            then
                sg_log 999 "WARNING: Not attempting to check DISKGROUP "\
                           "${ORACLE_ASM_DISKGROUP[$dg_index]} "
                to_exit=1
            else 
                if [[ "X${IS_FG_SYNC_IN_DG[$dg_index]}" = "X${FALSE}" ]];then 
                    if [[ "X${CLEAN_FG_IN_DG[$dg_index]}" = "X${FALSE}" ]];then
                        sg_log 0 "ERROR: Diskgroup \"${ORACLE_ASM_DISKGROUP[$dg_index]}\""\
                                 "was started with one failgroup on"\
                                 "the previous ocassion."
                        sg_log 0 "Preventing start of package again with one"\
                                 "failgroup in diskgroup \"${ORACLE_ASM_DISKGROUP[$dg_index]}\""
                        sg_log 0 "in order to protect data consistency."
                        sg_log 0 "******************* WARNING ***************************"
                        sg_log 0 "Forcing a package start, will result in major data loss"
                        sg_log 0 "if diskgroup \"${ORACLE_ASM_DISKGROUP[$dg_index]}\" is not started "\
                                 "with the same failgroup"
                        sg_log 0 "with which it was running on the previous node"
                        sg_log 0 "Execute the following commands to FORCE a package start:"
                        sg_log 0 "      rm $DG_STATUS_FILE"
                        sg_log 0 "      cmrunpkg <package_name>"
                        sg_log 0 "On the current node."
                        sg_log 0 "ERROR: Failed to start package \"$pkg\""\
                                 "as data on ASM diskgroup \"${ORACLE_ASM_DISKGROUP[$dg_index]}\""\
                                 "is not in sync and does not meet the"\
                                 "RPO criteria, exiting."
                        to_exit=1
                        # Below will set global flag so that all other 
                        # package module will get to know there is 
                        # a failure while reconfiguration (while adding 
                        # new module online).
                        if sg_check_if_it_is_reconfiguration
                        then
                            sg_set_reconfig_failure
                        fi
                        return
                    fi 
                fi
            fi                
            dg_index=$(($dg_index+1))
        done      
    fi 
    sg_log 0 "Oracle ASM mirror startup succeeded, as all configured"\
             "Oracle ASM diskgroups are meeting RPO criteria."
    to_exit=0
    return   
}

#######################################################################
#
# function ora_validate_asm_mirror() is called by master control script
# during Oracle package validation with ASM mirror.
#
#######################################################################

function ora_validate_asm_mirror
{
    typeset pkg=${SG_PACKAGE_NAME}
    typeset -i exit_code=0
    typeset -i return_code=0
    typeset -i is_cleanup_conf_status_file_required=$FALSE
    typeset -i is_asm_lib_drv_loaded=$FALSE
    typeset -i is_asm_fltr_drv_loaded=$FALSE
    sg_log 5 "Function ora_validate_asm_mirror"
    ora_mirror_get_monitor_service
    if [[ -z "${monitoring_service_name}" ]]
    then
        sg_log 0 "ERROR: Failed to validate package \"$pkg\" as ASM"\
                 "Mirror monitoring service is not configured"
        exit_code=1 
    fi
    if [[ "${ORA_INSTANCE_TYPE}" != "database" ]]
    then
        sg_log 0 "ERROR: Failed to validate package \"$pkg\" as"\
                 "instance type is other than "database"."
        if [[ $exit_code -eq 0 ]];then
            exit_code=1
        fi 
    fi
    
    # Validation fails if ASM parameter is not configured in package or 
    # set to no 
    if [[ "${ORA_ASM}" != "yes" ]]
    then
        sg_log 0 "ERROR: Failed to validate package \"$pkg\" as Oracle"\
                 "database does not use \"ASM\" as storage."
        if [[ $exit_code -eq 0 ]];then
            exit_code=1
        fi 
    fi

    # Validate setting for Oracle ASM Lib driver 
    check_asm_lib_driver_state 
    is_asm_lib_drv_loaded=$?
    if [[ $is_asm_lib_drv_loaded -eq 0 ]];then
        sg_log 0 "INFO: Oracle ASM Lib driver is configured"
        # Validate setting for Oracle ASM Lib driver 
        check_asm_lib_driver_access 
        if [[ $? -ne 0 ]];then
            exit_code=1
        fi 
    fi 

    # Validate Oracle ASM filter driver state
    check_asm_fltr_driver_state
    is_asm_fltr_drv_loaded=$?
    if [[ $is_asm_fltr_drv_loaded -eq 0 ]];then
        sg_log 0 "INFO: Oracle ASM filter driver is configured"
    fi 

    if [[ $is_asm_fltr_drv_loaded -eq $TRUE &&\
          $is_asm_lib_drv_loaded -eq $TRUE ]];then
        sg_log 0 "ERROR: Both Oracle ASM Lib driver and ASM filter driver"\
                 "are configured. Unconfigure one of them and try again"
        exit_code=1
    elif [[ $is_asm_fltr_drv_loaded -eq $FALSE &&\
            $is_asm_lib_drv_loaded -eq $FALSE ]];then
        sg_log 0 "ERROR: Neither Oracle ASM Lib driver nor ASM filter driver"\
                 "is configured. Configure one of them and try again"
        exit_code=1
    elif [[ "$SG_COMMAND" = "cmapplyconf" ]];then
        PREV_ORA_ASM_DRIVER_TYPE=`cat ${ASM_DRIVER_INFO_FILE} 2>/dev/null`
        if [[ $is_asm_fltr_drv_loaded -eq $TRUE ]];then 
            echo "AFD" > ${ASM_DRIVER_INFO_FILE}
            ORA_ASM_DRIVER_TYPE=`cat ${ASM_DRIVER_INFO_FILE}`
            if [[ -n "$PREV_ORA_ASM_DRIVER_TYPE" &&\
                 "$PREV_ORA_ASM_DRIVER_TYPE" !=\
                               "$ORA_ASM_DRIVER_TYPE" ]];then
                 is_cleanup_conf_status_file_required=$TRUE
            fi 
        elif [[ $is_asm_lib_drv_loaded -eq $TRUE ]];then
            echo "ALIB" > ${ASM_DRIVER_INFO_FILE}
            ORA_ASM_DRIVER_TYPE=`cat ${ASM_DRIVER_INFO_FILE}`
            if [[ -n "$PREV_ORA_ASM_DRIVER_TYPE" &&\
                 "$PREV_ORA_ASM_DRIVER_TYPE" !=\
                               "$ORA_ASM_DRIVER_TYPE" ]];then
                 is_cleanup_conf_status_file_required=$TRUE
            fi 
        else 
            sg_log 0 "ERROR: Unsupported ASM driver is configured"
            exit_code=1
        fi 
    fi 

    # If there is no issue on validating ASM Lib driver
    # or ASM filter driver, then only check for ASM_PV 
    # list change. 
    check_asm_pv_list_change
    if [[ $CHANGE_IN_ASM_PV_LIST -eq $TRUE ]];then
        if [[ "$SG_COMMAND" = "cmapplyconf" ]];then
            is_cleanup_conf_status_file_required=$TRUE
        fi 
    fi 
    
    # Only when there is a change in ASM driver or ASM_PV 
    # list change, if flag "is_cleanup_conf_state_file_required"
    # is set to "TRUE" then remove both STATUS and CONF files.
    if [[ $is_cleanup_conf_status_file_required -eq $TRUE ]];then
        cleanup_conf_status_file 
    fi 

    if [[ ! -d $ORA_ASM_MIR_STATUS_DIR ]];then
        mkdir -p $ORA_ASM_MIR_STATUS_DIR >/dev/null 2>&1
        if [[ $? -ne 0 ]];then
            sg_log 0 "ERROR: Unable to create directory"\
                     "\"$ORA_ASM_MIR_STATUS_DIR\""
            exit_code=1
        fi
    fi 
    to_exit=$exit_code
    return
}

#######################################################################
#
# function ora_asm_mirr_reconfig_stop_monitor_service() stops the ASM
# mirror monitoring service.
#
#######################################################################

function ora_asm_mirr_reconfig_stop_monitor_service
{
    sg_log 5 "Function ora_asm_mirr_reconfig_stop_monitor_service"
    if [[ -n "$monitoring_service_name" ]]
    then
        sg_log 0 "Stopping service ${monitoring_service_name}"
        ${SGSBIN}/cmhaltserv ${monitoring_service_name} \
                                     >> $SG_SCRIPT_LOG_FILE 2>&1 
        if [[ $? -ne 0 ]]
        then
            sg_log 999 "ERROR: Failed to halt the service"
            sg_log 999 "${monitoring_service_name} for online"\
                       "package modification"
            sg_set_reconfig_failure
            to_exit=1
        fi
    fi
}

#######################################################################
#
# function ora_asm_mirr_reconfig_start_monitor_service() starts the ASM
# mirror monitoring service.
#
#######################################################################

function ora_asm_mirr_reconfig_start_monitor_service
{
    sg_log 5 "Function ora_asm_mirr_reconfig_start_monitor_service"
    if [[ -n "${monitoring_service_name}" ]]
    then
        sg_log 0 "Starting service ${monitoring_service_name}"
        ${SGSBIN}/cmrunserv ${monitoring_service_name} \
        "${monitoring_service_cmd}" >> $SG_SCRIPT_LOG_FILE 2>&1
        if [[ $? -ne 0 ]]
        then
            sg_log 999 "ERROR: Failed to start the service"
            sg_log 999 "${monitoring_service_name} for online"\
                       "package modification"
            sg_set_reconfig_failure
            to_exit=1
        fi
    fi
}

#######################################################################
#
# function ora_asm_mirr_reconfig_restart_monitor_service() restarts the 
# Oracle ASM mirror monitoring service.
#
#######################################################################

function ora_asm_mirr_reconfig_restart_monitor_service
{
    sg_log 5 "Function ora_asm_mirr_reconfig_restart_monitor_service"
    if [[ -n "${monitoring_service_name}" ]]
    then
        if ! sg_check_if_reconfig_has_failed
        then
            ora_asm_mirr_reconfig_stop_monitor_service
            ora_asm_mirr_reconfig_start_monitor_service
            sg_log 0 "Successfully restarted service"\
                     "${monitoring_service_name}"
            sg_log 999 "Successfully restarted service"\
                       "${monitoring_service_name}"
        else 
            sg_log 999 "WARNING: Oracle toolkit - Not attempting"\
                       "to restart service"\
                       "\"${monitoring_service_name}\""
            # Restores the old ASM mirror conf file and dg status file 
            # if any failure occurs and ASM mirror conf file or dg 
            # status file does not restore then just show warning.
            restores_conf_status_file
            if [[ $? -ne 0 ]];then 
                sg_log 999 "WARNING: Unable to restore ASM mirror"\
                           "configuration file and disk group status"\
                           "file"
            fi 
        fi
    fi
    return
}

#######################################################################
#
# function ora_asm_mirr_reconfigure_stop() is called by master 
# control script during oracle asm package reconfigure_stop entry point.
#
#######################################################################

function ora_asm_mirr_reconfigure_stop
{
    sg_log 5 "Function ora_asm_mirr_reconfigure_stop"
    # To check is there any change in
    # ORACLE_ASM_PV list
    check_asm_pv_list_change
    if [[ "${PREV_ORA_ASM_MIRROR_MONITOR_INTERVAL}" != \
          "${ORA_ASM_MIRROR_MONITOR_INTERVAL}" ||\
          "${PREV_SG_SCRIPT_LOG_FILE}" != "${SG_SCRIPT_LOG_FILE}" ||\
          "${PREV_SG_EMAIL_ID}" != "${SG_EMAIL_ID}" ||\
          $CHANGE_IN_ASM_PV_LIST -eq $TRUE ]];then 
        cache_dg_info
        if [[ $? -ne 0 ]];then
            sg_set_reconfig_failure
            to_exit=1
        fi
    fi
}

#######################################################################
#
# function ora_asm_mirr_reconfigure_start() is called by master control 
# script during oracle asm package reconfigure_start entry point.
#
#######################################################################

function ora_asm_mirr_reconfigure_start
{
    sg_log 5 "Function ora_asm_mirr_reconfigure_start" 
    check_asm_pv_list_change
    if [[ $CHANGE_IN_ASM_PV_LIST -eq $TRUE ]];then
        # Repopulate all asm device related info like asm disk name,
        # diskgroup and failgroup as there is change in "ASM_PV" list
        populate_asm_dev_list_info "reconfigure_start"
        if [[ $? -ne 0 ]];then
            sg_set_reconfig_failure 
            to_exit=1
        fi
        ora_mirror_get_monitor_service
        ora_asm_mirr_reconfig_restart_monitor_service
    fi
}

#######################################################################
#
# function check_asm_pv_list_change() is called by master control 
# script for any operation on package, this will set a global flag 
# (CHANGE_IN_ASM_PV_LIST), if any change in ASM_PV list.
#
#######################################################################

function check_asm_pv_list_change
{
    sg_log 5 "Function check_asm_pv_list_change" 

    if [ "$PKG_NAME" = "" ]
    then
        PKG_NAME=$SG_PACKAGE_NAME
    fi
    for I in ${ORACLE_ASM_PV[*]}
    do
        found=0
        for J in ${PREV_ORACLE_ASM_PV[*]}
        do
            if [[ $I = $J ]]
            then
                found=1
                break;
            fi
        done
        if [[ $found -eq 0 ]]
        then
            if ! sg_check_if_reconfig_has_failed
            then
                if [[ "$OP" = "reconfigure_start" ]];then
                    sg_log 0 "New ASM_PV $I added"\
                             "to package \"$PKG_NAME\""
                fi 
                CHANGE_IN_ASM_PV_LIST=$TRUE
            fi
        fi
    done
    for I in ${PREV_ORACLE_ASM_PV[*]}
    do
        found=0
        for J in ${ORACLE_ASM_PV[*]}
        do
            if [[ $I = $J ]]
            then
                found=1
                break
            fi
        done
        if [[ $found -eq 0 ]]
        then
            if ! sg_check_if_reconfig_has_failed
            then
                if [[ "$OP" = "reconfigure_stop" ]];then
                    sg_log 0 "ASM PV removed $I"\
                             "from package \"$PKG_NAME\""
                fi 
                CHANGE_IN_ASM_PV_LIST=$TRUE
            fi
        fi
    done
}

################
# main routine
################

#
# Module script must be specified with following required entry points:
# start, stop, validate, reconfigure_start and reconfigure_stop.
#
# The variable to_exit indicates the success or failure of the entry
# point.
#

typeset -i to_exit=0
typeset OP=${1}

sg_log 5 "Oracle ASM mirror Module Script"
if [[ "X${1}" == "Xvalidate" ]];then
    sg_log 5 "*** $0 called with $1 argument. ***"
else 
    if [[ -f ${ASM_DRIVER_INFO_FILE} ]];then 
        ORA_ASM_DRIVER_TYPE=`cat ${ASM_DRIVER_INFO_FILE}`
    fi 
    sg_log 0 "*** $0 called with $1 argument. ***"
fi 

case ${1} in
    start)
        ora_start_asm_mirror
        ;;
    stop)
        ;;
    validate)
        ora_validate_asm_mirror
        ;;
    reconfigure_start)
        ora_asm_mirr_reconfigure_start
        ;;
    reconfigure_stop)
        ora_asm_mirr_reconfigure_stop
        ;;
    *)
        sg_log 0 "INFO: Unknown operation - $1 "
        to_exit=0          
        ;;
esac

#
# Exit to indicate success/failure to the master_control_script.sh
#
exit $to_exit
