#!/bin/bash
# (c) Copyright 2016 Hewlett Packard Enterprise Development LP.
# (c) Copyright 2007-2016 Hewlett Packard Development Company, L.P.
# Version A.12.00.51
#############################################
#    Linux ORACLE TOOLKIT MODULE SCRIPT     #  
#############################################

##########################################################################
# REVISION HISTORY:
#     Version        mm/dd/yyyy   Changes
#     A.05.01        04/30/2012   1) Added code to support Multiple Listeners. 
#                                 2) Added Code to support database hang detection.
#
#     A.05.01.01     10/12/2012   1) Added code to support online package
#                                    modification.                   
#     A.05.01.02     03/20/2013   1) Corrected numeric comparison operators.
#     A.05.01.10     04/18/2013   1) Added code to support cluster verification.
#                                 2) Modified the code to restrict online
#                                    update of attributes which can not be
#                                    modified when package is up and running.
#                                 3) Modified the code to restart the services
#                                    only when such attributes are modified.
#     A.05.01.11     07/22/2013   1) Added "-p $PACKAGE" instead of "grep $PACKAGE"
#                                 2) Added check for shared VG for ORACLE_HOME
#                                    & ASM_HOME in validate. Oracle binary/files
#                                    check is not done if shared vg is used
#     A.12.00.00     11/29/2013   1) Changes to obsolete legacy style of
#                                    packaging.
#                                 2) Added changes to check if online reconfiguration
#                                    for previous module has failed or not.
#                                 3) Added changes to check the presence of listener.ora
#                                    file in the TNS_ADMIN location, if TNS_ADMIN
#                                    variable is configured in customer.conf file.
#                               
#     A.12.00.10     07/18/2014   1) Modified code to remove validation call during 
#                                    package startup.
#     A.12.00.20     08/14/2014   1) Enhanced Error and Warning messages to support 
#                                    Error handling feature of Servicguard Manager.
#     A.12.00.40     08/31/2015   1) Fix for QXCR1001430463 
#     A.12.00.50     03/02/2016   1) Added changes to support online modification
#                                    of CRITICAL_PDB attribute.
###########################################################################
###########################
# Source utility functions.
###########################

 . /etc/cmcluster.conf
if [[ -z "$SG_UTILS" ]]
then
    SG_UTILS=$SGCONF/scripts/mscripts/utils.sh
fi

if [[ -f ${SG_UTILS} ]]; then
    . ${SG_UTILS}
    if [[ $? -ne 0 ]]
    then
        echo "ERROR: Unable to source package utility functions file: ${SG_UTILS}"
        exit 1
    fi
else
    echo "ERROR: Unable to find package utility functions file: ${SG_UTILS}"
    exit 1
fi

###################################################################
# Get the environment for this package through utility function
# sg_source_pkg_env().
#################################################################################

export PATH=$PATH:$SGSBIN


if [[ ! -f $2 ]] ; then
        sg_source_pkg_env $1
else
        sg_source_pkg_env $*
fi

#Source the haoracle_utils.sh
ORACLETKIT_UTILS="${SGROOT}/oracletoolkit/haoracle_utils.sh"
if [[ -f  $ORACLETKIT_UTILS ]]; then
    . ${ORACLETKIT_UTILS}
    if [[ $? -ne 0 ]]
    then
        echo "ERROR: Unable to source Oracle toolkit utility functions file: ${ORACLETKIT_UTILS}"
        exit 1
    fi
else
    echo "ERROR: Unable to find Oracle toolkit utility functions file: ${ORACLETKIT_UTILS}"
    exit 1
fi
#############################
#      START FUNCTION
#############################

function start_function
{
# Checks to see if this is online addition of new module

   if sg_check_if_reconfig_has_failed ; then
      sg_log 999 "WARNING: Oracle toolkit - Not attempting to start the newly added Oracle toolkit module"
      return 
   fi

   . ${TKIT_SCRIPT_DIR}/toolkit.sh start
   exit_val=$?
   if sg_check_if_it_is_reconfiguration ; then
     if [[  $exit_val -ne 0 ]]; then
       sg_log 999 "ERROR: Failed to start the newly added Oracle toolkit module"
       sg_set_reconfig_failure
     else    
       sg_log 999 "SUCCESS: Successfully started newly added Oracle toolkit module"
       sg_log 0 "SUCCESS: Successfully started newly added Oracle toolkit module"
     fi
   fi
}

####################################
# critical_pdb_verification function
####################################
function critical_pdb_verification
{
     declare -a pdb_list
     typeset -i exit_code=0
     typeset -i i=0
     for var in ${ORACLE_CRITICAL_PDB[@]}
     do
         pdb_list[$i]=`echo $var|tr '[[:upper:]]' '[[:lower:]]'`
         ((i=i+1))
     done
     if [[ "$START_MODE" = "open" ]]; then
        CDB=$(determine_db_type)
        if [[ "$CDB" = "YES" && "${pdb_list}" != "none" && "${pdb_list}" != "any" ]]; then
        for j in ${ORACLE_CRITICAL_PDB[@]}
        do
          check_if_pdb_exists $j
          if [[ $? -eq 1 ]]; then
             sg_log 0 "ERROR: Specified Critical PDB $j does not exists in the Container Database."
             exit_code=1
          fi
        done
        fi
     fi
     return $exit_code
}

#############################
#     MONITOR FUNCTION
#############################

function monitor_function
{
    . ${TKIT_SCRIPT_DIR}/toolkit.sh monitor
    exit_val=$?
}


#############################
# LISTENER MONITOR FUNCTION
#############################

function monitor_listener_function
{
    . ${TKIT_SCRIPT_DIR}/toolkit.sh monitor_listener $1
    exit_val=$?
}

###################################
# DATABASE HANG DETECTION FUNCTION
###################################

function monitor_database_hang
{
    . ${TKIT_SCRIPT_DIR}/toolkit.sh hang_monitor $1 $2
    exit_val=$?
}


#####################################
# PLUGGABLE DATABASE MONITOR FUNCTION
#####################################

function monitor_pdb
{
    . ${TKIT_SCRIPT_DIR}/toolkit.sh monitor_pdb
    exit_val=$?
}

#############################
#     STOP FUNCTION
#############################

function stop_function
{
     if [[ "$SG_HALT_REASON" = "user_halt" ]]; then
          reason="user"
     else
          reason="auto"
     fi
    . ${TKIT_SCRIPT_DIR}/toolkit.sh stop $reason
    exit_val=$?
}

############################################################
# Function to verify whether service restart is needed
# or not.
# Return values:
#   0 - if restart is needed
#   1 - if restart is not needed
############################################################

function verify_service_restart
{
cur_listener_name=( $(
   for list in "${ORACLE_LISTENER_NAME[@]}"
   do
      $ECHO "$list"
   done | $SORT) )

prev_listener_name=( $(
   for list in "${PREV_ORACLE_LISTENER_NAME[@]}"
   do
      $ECHO "$list"
   done | $SORT) )

cur_listener_pass=( $(
   for list in "${ORACLE_LISTENER_PASS[@]}"
   do
      $ECHO "$list"
   done | $SORT) )

prev_listener_pass=( $(
   for list in "${PREV_ORACLE_LISTENER_PASS[@]}"
   do
      $ECHO "$list"
   done | $SORT) )

cur_listener_restart=( $(
   for list in "${ORACLE_LISTENER_RESTART[@]}"
   do
      $ECHO "$list"
   done | $SORT) )

prev_listener_restart=( $(
   for list in "${PREV_ORACLE_LISTENER_RESTART[@]}"
   do
      $ECHO "$list"
   done | $SORT) )

cur_monitor_process=( $(
   for list in "${ORACLE_MONITOR_PROCESSES[@]}"
   do
      $ECHO "$list"
   done | $SORT) )

prev_monitor_process=( $(
   for list in "${PREV_ORACLE_MONITOR_PROCESSES[@]}"
   do
      $ECHO "$list"
   done | $SORT) )

cur_email_id=( $(
   for list in "${SG_EMAIL_ID[@]}"
   do
      $ECHO "$list"
   done | $SORT) )

prev_email_id=( $(
   for list in "${PREV_SG_EMAIL_ID[@]}"
   do
      $ECHO "$list"
   done | $SORT) )

cur_critical_pdb_name=( $(
   for list in "${ORACLE_CRITICAL_PDB[@]}"
   do
      $ECHO "$list"
   done | $SORT) )

prev_ciritical_pdb_name=( $(
   for list in "${PREV_ORACLE_CRITICAL_PDB[@]}"
   do
      $ECHO "$list"
   done | $SORT) )

if [[ "${ORACLE_LISTENER}" != "${PREV_ORACLE_LISTENER}" || "${cur_listener_name[*]}" != "${prev_listener_name[*]}" || "${cur_listener_pass[*]}" != "${prev_listener_pass[*]}" || "${cur_listener_restart[*]}" != "${prev_listener_restart[*]}" || "${ORACLE_PARENT_ENVIRONMENT}" != "${PREV_ORACLE_PARENT_ENVIRONMENT}" ]]; then
   listener_srv_restart=1
fi

if [[ "${cur_critical_pdb_name[*]}" != "${prev_ciritical_pdb_name[*]}" ]]; then
   db_pdb_srv_restart=1
fi

if [[ "${cur_monitor_process[*]}" != "${prev_monitor_process[*]}" ]]; then
   db_srv_restart=1
fi
if [[ "${ORACLE_MONITOR_INTERVAL}" != "${PREV_ORACLE_MONITOR_INTERVAL}" || "${cur_email_id[*]}" != "${prev_email_id[*]}" ]]; then
   all_srv_restart=1
fi

if [[ $listener_srv_restart -eq 1 || $db_srv_restart -eq 1 || $all_srv_restart -eq 1 || $db_pdb_srv_restart -eq 1 ]]; then
   return 0
else
   return 1
fi
}

############################################################
#
#       RECONFIGURE STOP FUNCTION 
#
############################################################


function reconfigure_stop_function
{
if ! sg_check_if_reconfig_has_failed
then 
  # check whether service restart is needed or not
  verify_service_restart
  if [[ $? -eq 0 ]]; then
    if [[ $all_srv_restart -eq 1 ]]; then
      for (( i=0; i<${#PREV_SG_SERVICE_NAME[@]}; i++ ))
      do
         ${SGSBIN}/cmhaltserv ${PREV_SG_SERVICE_NAME[$i]}
         if [[ $? -ne 0 ]]; then
            sg_log 999 "ERROR: Failed to stop the Oracle toolkit service $service_name"
            sg_set_reconfig_failure
            exit_val=1
         else
            sg_log 999 "SUCCESS: Stopping Oracle toolkit service $service_name success."
            sg_log 0 "SUCCESS: Stopping Oracle toolkit service $service_name success."
         fi
      done
    else
      if [[ "${PREV_ORACLE_LISTENER}" = "yes" || "${PREV_ORACLE_LISTENER}" = "Yes" || "${PREV_ORACLE_LISTENER}" = "YES" ]] && [[ $listener_srv_restart -eq 1 ]]; then
         for (( i=0; i<${#PREV_SG_SERVICE_NAME[@]}; i++ ))
         do
            if [[ "`$ECHO ${SG_SERVICE_CMD[$i]} | $AWK '{print $2}'`" = "oracle_monitor_listener" ]]; then
               ${SGSBIN}/cmhaltserv ${PREV_SG_SERVICE_NAME[$i]}
               if [[ $? -ne 0 ]]; then
                 sg_log 999 "ERROR: Failed to stop the Oracle toolkit service $service_name"
                 sg_set_reconfig_failure
                 exit_val=1
               else
                 sg_log 999 "SUCCESS: Stopping Oracle toolkit service $service_name success."
                 sg_log 0 "SUCCESS: Stopping Oracle toolkit service $service_name success."
               fi
            fi
         done
      elif [[ $db_srv_restart -eq 1 ]]; then
         for (( i=0; i<${#PREV_SG_SERVICE_NAME[@]}; i++ ))
         do
            if [[ "`$ECHO ${SG_SERVICE_CMD[$i]} | $AWK '{print $2}'`" = "oracle_monitor" ]]; then
               ${SGSBIN}/cmhaltserv ${PREV_SG_SERVICE_NAME[$i]}
               if [[ $? -ne 0 ]]; then
                  sg_log 999 "ERROR: Failed to stop the Oracle toolkit service $service_name"
                  sg_set_reconfig_failure
                  exit_val=1
               else
                  sg_log 999 "SUCCESS: Stopping Oracle toolkit service $service_name success."
                  sg_log 0 "SUCCESS: Stopping Oracle toolkit service $service_name success."
               fi
            fi
         done
      elif [[ $db_pdb_srv_restart -eq 1 ]]; then
         for (( i=0; i<${#PREV_SG_SERVICE_NAME[@]}; i++ ))
         do
           if [[ "`$ECHO ${SG_SERVICE_CMD[$i]} | $AWK '{print $2}'`" = "oracle_pdb_monitor" ]]; then
              ${SGSBIN}/cmhaltserv ${PREV_SG_SERVICE_NAME[$i]}
              if [[ $? -ne 0 ]]; then
                 sg_log 999 "ERROR: Failed to stop the Oracle toolkit service $service_name"
                 sg_set_reconfig_failure
		 exit_val=1
	      else
		 sg_log 999 "SUCCESS: Stopping Oracle toolkit service $service_name success."
                 sg_log 0 "SUCCESS: Stopping Oracle toolkit service $service_name success."
             fi
          fi
         done
      fi
  fi
else
  sg_log 999 "WARNING: Oracle toolkit - Not attempting to stop any oracle toolkit service"
fi
fi
}

############################################################
#
#       RECONFIGURE START FUNCTION 
#
############################################################

function reconfigure_start_function
{
   # check whether service restart is needed or not
  verify_service_restart
  if [[ $? -eq 0 ]]; then
    if [[ $all_srv_restart -eq 1 ]]; then
      for (( i=0; i<${#SG_SERVICE_NAME[@]}; i++ ))
      do
         msg="cmrunserv: Unable to start service ${SG_SERVICE_NAME[$i]} (it is already active)"
         out=`${SGSBIN}/cmrunserv ${SG_SERVICE_NAME[$i]} "${SG_SERVICE_CMD[$i]}" 2>&1`
         ret=$? 
         if [[ $ret -ne 0 ]] && [[ "$out" != "$msg" ]]; then
            sg_log 999 "ERROR: Failed to start the Oracle toolkit service ${SG_SERVICE_NAME[$i]}"
            sg_set_reconfig_failure 
            exit_val=1
         elif [[ $ret -eq 0 ]]; then
            sg_log 999 "SUCCESS: Successfully started Oracle toolkit service ${SG_SERVICE_NAME[$i]}."
            if ! sg_check_if_reconfig_has_failed ; then
              sg_log 0 "SUCCESS: Successfully started Oracle toolkit service ${SG_SERVICE_NAME[$i]}."
            fi
         fi

       done
else
      if [[ "${ORACLE_LISTENER}" = "yes" || "${PREV_ORACLE_LISTENER}" = "Yes" || "${PREV_ORACLE_LISTENER}" = "YES" ]] && [[ $listener_srv_restart -eq 1 ]]; then
         for (( i=0; i<${#SG_SERVICE_NAME[@]}; i++ ))
         do
            if [[ "`$ECHO ${SG_SERVICE_CMD[$i]} | $AWK '{print $2}'`" = "oracle_monitor_listener" ]]; then
               msg="cmrunserv: Unable to start service ${SG_SERVICE_NAME[$i]} (it is already active)"
               out=`${SGSBIN}/cmrunserv ${SG_SERVICE_NAME[$i]} "${SG_SERVICE_CMD[$i]}" 2>&1`
               ret=$?
               if [[ $ret -ne 0 ]] && [[ "$out" != "$msg" ]]; then
                  sg_log 999 "ERROR: Failed to start the Oracle toolkit service ${SG_SERVICE_NAME[$i]}"
                  sg_set_reconfig_failure 
                  exit_val=1
               elif [[ $ret -eq 0 ]]; then
                  sg_log 999 "SUCCESS: Successfully started Oracle toolkit service ${SG_SERVICE_NAME[$i]}"
                  if ! sg_check_if_reconfig_has_failed ; then
                     sg_log 0 "SUCCESS: Successfully started Oracle toolkit service ${SG_SERVICE_NAME[$i]}."
                  fi
               fi
            fi
         done
      elif [[ $db_srv_restart -eq 1 ]]; then
         for (( i=0; i<${#SG_SERVICE_NAME[@]}; i++ ))
         do
            if [[ "`$ECHO ${SG_SERVICE_CMD[$i]} | $AWK '{print $2}'`" = "oracle_monitor" ]]; then
               msg="cmrunserv: Unable to start service ${SG_SERVICE_NAME[$i]} (it is already active)"
               out=`${SGSBIN}/cmrunserv ${SG_SERVICE_NAME[$i]} "${SG_SERVICE_CMD[$i]}" 2>&1`
               ret=$?
               if [[ $ret -ne 0 ]] && [[ "$out" != "$msg" ]]; then
                  sg_log 999 "ERROR: Failed to start the Oracle toolkit service ${SG_SERVICE_NAME[$i]}"
                  sg_set_reconfig_failure 
                  exit_val=1
               elif [[ $ret -eq 0 ]]; then
                  sg_log 999 "SUCCESS: Successfully started Oracle toolkit service ${SG_SERVICE_NAME[$i]}"
                  if ! sg_check_if_reconfig_has_failed ; then
                     sg_log 0 "SUCCESS: Successfully started Oracle toolkit service ${SG_SERVICE_NAME[$i]}."
                  fi
               fi
            fi
         done
        elif [[ $db_pdb_srv_restart -eq 1 ]]; then
           for (( i=0; i<${#SG_SERVICE_NAME[@]}; i++ ))
           do
               if [[ "`$ECHO ${SG_SERVICE_CMD[$i]} | $AWK '{print $2}'`" = "oracle_pdb_monitor" ]]; then
                  msg="cmrunserv: Unable to start service ${SG_SERVICE_NAME[$i]} (it is already active)"
                  out=`${SGSBIN}/cmrunserv ${SG_SERVICE_NAME[$i]} "${SG_SERVICE_CMD[$i]}" 2>&1`
                  ret=$?
                  if [[ $ret -ne 0 ]] && [[ "$out" != "$msg" ]]; then
                     sg_log 999 "ERROR: Failed to start the Oracle toolkit service ${SG_SERVICE_NAME[$i]}"
                     sg_set_reconfig_failure
                     exit_val=1
                  elif [[ $ret -eq 0 ]]; then
                       sg_log 999 "SUCCESS: Successfully started Oracle toolkit service ${SG_SERVICE_NAME[$i]}"
                       if ! sg_check_if_reconfig_has_failed ; then
                          sg_log 0 "SUCCESS: Successfully started Oracle toolkit service ${SG_SERVICE_NAME[$i]}."
                       fi
                  fi
               fi
           done
      fi
    fi
  fi
} 
#############################
# Verify whether unsupported toolkit parameters are being changed during
# online reconfiguration. If any of the unsupported attributes being
# modified then function will exit.
#############################

function verify_unsupported_online_parms
{
   # array of unsupported Oracle toolkit package attributes
   declare -a unsupported_attr=("INSTANCE_TYPE" "ORACLE_HOME" "ORACLE_ADMIN" "SID_NAME" "ASM" "ASM_DISKGROUP" "ASM_VOLUME_GROUP" "ASM_HOME" "ASM_USER" "ASM_SID")

   # array of unsupported Oracle toolkit package attributes values from the ascii file
   declare -a cur_attr_values=("$ORACLE_INSTANCE_TYPE" "$ORACLE_ORACLE_HOME" "$ORACLE_ORACLE_ADMIN" "$ORACLE_SID_NAME" "$ORACLE_ASM" "$ORACLE_ASM_DISKGROUP" "$ORACLE_ASM_VOLUME_GROUP" "$ORACLE_ASM_HOME" "$ORACLE_ASM_USER" "$ORACLE_ASM_SID")

   # array of Oracle toolkit package attributes values of running package
   declare -a prev_attr_values=("$PREV_ORACLE_INSTANCE_TYPE" "$PREV_ORACLE_ORACLE_HOME" "$PREV_ORACLE_ORACLE_ADMIN" "$PREV_ORACLE_SID_NAME" "$PREV_ORACLE_ASM" "$PREV_ORACLE_ASM_DISKGROUP" "$PREV_ORACLE_ASM_VOLUME_GROUP" "$PREV_ORACLE_ASM_HOME" "$PREV_ORACLE_ASM_USER" "$PREV_ORACLE_ASM_SID")

   unsupported_flag=0

   for (( i=0; i<${#unsupported_attr[@]}; i++ ))
   do
      if [[ "${cur_attr_values[$i]}" != "${prev_attr_values[$i]}" ]]; then
            sg_log 0 "ERROR: Online modification of attribute \"${unsupported_attr[$i]}\" is not allowed."
            unsupported_flag=1
      fi
   done

   if [[ $unsupported_flag -eq 1 ]]; then
      exit 1
   fi
}

#############################
#   ORACLE FILES VALIDATIONS
#############################

function check_ora_files
{
       err_code=0
       retval=1

       # if LISTENER is set to yes then check existence of listener.ora on all package nodes
       if [[ "${ORACLE_LISTENER}" = "yes" || "${ORACLE_LISTENER}" = "YES" || "${ORACLE_LISTENER}" = "Yes" ]]; then
         #if TNS_ADMIN variable is defined in ${ORACLE_SID_NAME}_customer.conf, determine the location of the listener.ora using ${ORACLE_SID_NAME}_customer.conf
           if [[ -f $TKIT_SCRIPT_DIR/${ORACLE_SID_NAME}_customer.conf ]]; then
              tnsadmin_loc=`cat $TKIT_SCRIPT_DIR/${ORACLE_SID_NAME}_customer.conf | grep "TNS_ADMIN=" | cut -d "=" -f 2`
              retval=$?
              if [[ $retval -eq 0 && -n "$tnsadmin_loc" ]]; then
                 LIST_LOC="$tnsadmin_loc"
              elif [[  $retval -ne 0 || -z "$tnsadmin_loc" ]]; then
                 LIST_LOC="$ORACLE_HOME/network/admin" 
              fi
              if [[ ! -e "$LIST_LOC/listener.ora"  ]]; then
                 sg_log 0 "ERROR: Check listener file path defined for TNS_ADMIN in ${ORACLE_SID_NAME}_customer.conf file."
              fi
           else
                LIST_LOC="$ORACLE_HOME/network/admin"
           fi

          if [[ ! -e "$LIST_LOC/listener.ora"  ]]; then
             sg_log 0 "ERROR: Listener file ${LIST_LOC}/listener.ora is missing."
             err_code=1
          fi
       fi

       # check for the PFILE existance
       if [[ ! -e ${PFILE} ]]; then
          sg_log 0 "WARNING: Oracle toolkit - Oracle Parameter File (PFILE) ${PFILE} is missing."
       fi
    
       # check for the existance of sqlplus command
       if [[ ! -f $ORACLE_ORACLE_HOME/bin/sqlplus ]]; then
          sg_log 0 "ERROR: The Oracle command "$ORACLE_ORACLE_HOME/bin/sqlplus" is missing."
          err_code=1
       fi

       # check for the existance of opatch command
       if [[ ! -f $ORACLE_ORACLE_HOME/OPatch/opatch ]]; then
          sg_log 0 "ERROR: The Oracle command "$ORACLE_ORACLE_HOME/OPatch/opatch" is missing."
          err_code=1
       fi

       if [[ $err_code -eq 1 ]] ; then
             return 1
       else
             return 0
       fi

}

#############################
#    VALIDATE FUNCTION
#############################

function validate_function
{
    sg_log 0 "Oracle Toolkit validation"

    # Validation for ATX
    # User can start the Oracle listener instance using ATX.
    if [[ -n "$PEV_ATX_ORACLE_LSNR" ]]; then
        sg_validate_atx
        if [[ $? -ne 0 ]]; then
            exit_code=1
        fi

        atx_listener_log_file_flag=''
        PEV_ATX_LISTENER_Array=($PEV_ATX_ORACLE_LSNR)
        for atx_parm_val in "${!PEV_ATX_LISTENER_Array[@]}";
        do
            if [ "${PEV_ATX_LISTENER_Array[$atx_parm_val]}" = "-l" ] || [ "${PEV_ATX_LISTENER_Array[$atx_parm_val]}" = "--log" ]; then
                atx_log_index=${atx_parm_val}
                ((atx_log_index=atx_log_index+1))
                if [[ "${PEV_ATX_LISTENER_Array[$atx_parm_val]}" = "-l" ]]; then
                    atx_listener_log_file="${PEV_ATX_LISTENER_Array[$atx_log_index]}"
                else
                    atx_listener_log_file="${PEV_ATX_LISTENER_Array[$atx_log_index]}"
                fi
                atx_listener_log_file_flag='enabled'
            fi
        done

        # Check whether defined ATX parameters are correct or not
        if [ ! -z "$atx_listener_log_file_flag" ]; then
            ATX_LOG_DIR=$(dirname "${atx_listener_log_file}")
            su ${ORACLE_ADMIN} -c '/bin/bash' <<EOF
            mkdir -p $ATX_LOG_DIR > /dev/null 2>&1
            if [[ \$? -eq 0 ]]; then
                rm -f ${atx_listener_log_file} > /dev/null 2>&1
                touch ${atx_listener_log_file} > /dev/null 2>&1
                if [[ \$? -ne 0 ]]; then
                    exit 1
                fi
            else
                exit 1
            fi
EOF
            if [[ $? -ne 0 ]]; then
                echo "ERROR: ${ORACLE_ADMIN} user doesn't have write permission on directory $ATX_LOG_DIR. Please check the defined \"pev_atx_oracle_lsnr\" parameter"
                exit_code=1
            fi
        fi

        # Check whether defined ATX parameters are correct or not
        HPE_ATX_LISTENER_CMD="/usr/bin/hpe-atx ${PEV_ATX_ORACLE_LSNR}"
        ${HPE_ATX_LISTENER_CMD} ls 1> /dev/null
        if [[ $? -ne 0 ]]; then
            sg_log 0 "ERROR: hpe-atx execution failed. Please check the parameters defined"
            exit_code=1
        fi
        rm -f ${atx_listener_log_file} > /dev/null 2>&1
    fi

    typeset -i exit_status=0
    # Cluster verification and restricting online modification unsupported
    # toolkit parameter features will be enabled only if Serviceguard version
    # A.11.20.20 or later is installed.
    if [[ "$sg_version" > "11.20.20" || "$sg_version" = "11.20.20" ]];  then

       # if command "cmcheckconf -P" and command "cmapplyconf"
       if [[ -z "$SG_PRIV_CLUSTER_VERIFICATION"  || "$SG_COMMAND" = "cmapplyconf" ]]; then

          # invoke verify_unsupported_online_parms function only if package already exists and
          # it is up and running
          ${SGSBIN}/cmviewcl -f line -s config -p $SG_PACKAGE >/dev/null 2>&1
          if [[ $? -eq 0 ]]; then

             # Check if cmapply command is run when package is up
             checkpackage_up=`${SGSBIN}/cmviewcl -f line -p $SG_PACKAGE | $GREP status | $CUT -d "=" -f 2`
            
             # Check if package is running and check if this modules is not newly getting
             # added during online configuration, then only call function verify_unsupported_online_modification
 
             if [[ "$checkpackage_up" = "up" && "$op_code" != "start" ]]; then
               if [[ ${PREV_SG_MODULE_NAME[@]} = *"tkit/oracle/oracle"* ]]; then
                verify_unsupported_online_parms
               fi
               critical_pdb_verification
               if [[ $? -ne 0 ]]; then
                 exit_code=1
               fi
             fi
          fi
       fi

       sid_name=${ORACLE_SID_NAME}
       asm_sid_name=${ORACLE_ASM_SID}

       # check for the consistency of the package attributes
       if [[ "${ORACLE_LISTENER}" = "no" || "${ORACLE_LISTENER}" = "NO" || "${ORACLE_LISTENER}" = "No" ]]; then
          for (( i=0; i<${#SG_SERVICE_CMD[@]}; i++ ))
          do
             # check for the mention of listener service command
             if [[ `$ECHO ${SG_SERVICE_CMD[$i]} | $AWK '{print $2}'` = "oracle_monitor_listener" ]]; then
                sg_log 0 "ERROR: The LISTENER parameter has been set to \"no\" but the service_cmd has \
still the listener service \"oracle_listener_service\". Comment out the \"oracle_listener_service\" service."
                exit_code=1
             fi
          done
       fi

       # Checking for existance Oracle user
       $ID_CMD $ORACLE_ORACLE_ADMIN > /dev/null 2>&1
       if [[ $? -ne 0 ]]; then
          sg_log 0 "ERROR: Oracle Admin \"$ORACLE_ORACLE_ADMIN\" doesn't exist."
          exit_code=1
       fi

       # Checking for existance Oracle ASM user
       if [[ "${ORACLE_INSTANCE_TYPE}" = "ASM" ]]; then
          $ID_CMD $ORACLE_ASM_USER > /dev/null 2>&1
          if [[ $? -ne 0 ]]; then
             sg_log 0 "ERROR: ASM User \"$ORACLE_ASM_USER\" doesn't exist."
             exit_code=1
          fi

          # Search through FS directory to see if it has ASM binary vg/fs in it
          asm_bin_shared=false
          for (( i=0; i<${#SG_FS_DIRECTORY[@]}; i++)) ; do
              if [[ -n "${SG_FS_DIRECTORY[@]}" ]] ; then
                 if $ECHO ${ORACLE_ASM_HOME} | $GREP -q -w "^${SG_FS_DIRECTORY[$i]}" 2>/dev/null ; then
                    asm_bin_shared=true
                    break
                 fi
              fi
          done
       fi

       ora_bin_shared=false
       # Search through FS directory to see if it has oracle binary vg/fs in it
       if [[ -n "${SG_FS_DIRECTORY[@]}" ]] ; then
          for (( i=0; i<${#SG_FS_DIRECTORY[@]}; i++)) ; do
              if $ECHO ${ORACLE_ORACLE_HOME} | $GREP -q -w "^${SG_FS_DIRECTORY[$i]}" 2>/dev/null ; then
                 ora_bin_shared=true
                 break
              fi
          done
       fi
 
       # Validate Oracle binary/files existence during startup or only if Oracel binaries/files are
       # not placed in shared storage. Even if Oracle binaries/files are on local/non-shared vg, skip
       # executing check_ora_files()
       if [[ $op_code = "start" ]] || ! ${ora_bin_shared} ; then
       # Checking oracle files existence
           if check_ora_files ; then
              exit_status=0
           else
              exit_status=1
           fi
       fi
       # Check if CRITICAL_PDB' attribute is specified for Oracle version
       # which are not 12c.
       if [[ -n "${ORACLE_CRITICAL_PDB[*]}" ]]; then
            oracle_version=`check_oracle_version`
            if [[ "$oracle_version" = "11gR1" ]] || [[ "$oracle_version" = "11gR2" || "$oracle_version" = "10gR" ]] ; then
               sg_log 0 "ERROR: Specifying critical PDB is not supported with Oracle version lesser than 12cR1"
               exit_code=1
            fi
       fi
  
       # This section is invoked when cmcheckconf -v command has been invoked.
       # In this section warning validations been placed.
       if [[ "$SG_PRIV_CLUSTER_VERIFICATION" = "y" ]]; then

          # check if ORACLE_HOME environment variable has been set for
          # the oracle user or not
          $SU_CMD - ${ORACLE_ORACLE_ADMIN} -s /bin/bash <<EOF
          $ENV_CMD | $GREP -q ORACLE_HOME
EOF
          if [[ $? -ne 0 ]]; then
             sg_log 0 "WARNING: Oracle toolkit - ORACLE_HOME environment variable is not set for \"${ORACLE_ORACLE_ADMIN}\" user."
          fi

          # check if ORACLE_SID environment variable has been set for
          # the oracle user or not
          $SU_CMD - ${ORACLE_ORACLE_ADMIN} -s /bin/bash <<EOF
          $ENV_CMD | $GREP -q ORACLE_SID
EOF
          if [[ $? -ne 0 ]]; then
             sg_log 0 "WARNING: Oracle toolkit - ORACLE_SID environment variable is not set for \"${ORACLE_ORACLE_ADMIN}\" user."
          fi
       fi
       
       if [[ $exit_code -ne 0 ]] || [[ $exit_status -ne 0 ]]; then
           exit_val=1
       else
           exit_val=0
       fi
    fi # End of sg_version check for "11.20.20"
    return $exit_val
}

 
############################
#    START OF ROUTINES
############################

typeset -i exit_val=0
typeset -i listener_srv_restart=0
typeset -i db_srv_restart=0
typeset -i all_srv_restart=0
export TKIT_SCRIPT_DIR=$SGROOT/oracletoolkit
CAT=/bin/cat
ECHO=/bin/echo
GREP=/bin/grep
AWK=/bin/awk
CUT=/usr/bin/cut
HOST_CMD=/bin/hostname
PASSWD_FILE=/etc/passwd
RPM_CMD=/bin/rpm
ENV_CMD=/usr/bin/env
SU_CMD=/bin/su
ID_CMD=/usr/bin/id
SORT=/bin/sort
# Assign the Serviceguard version
sg_version=`$SGSBIN/cmversion | $CUT -d "." -f 2-4`
export INSTANCE_TYPE=$ORACLE_INSTANCE_TYPE
export ORACLE_HOME=$ORACLE_ORACLE_HOME
export ORACLE_ADMIN=$ORACLE_ORACLE_ADMIN
export SID_NAME=$ORACLE_SID_NAME
export START_MODE=$ORACLE_START_MODE
export LISTENER=$ORACLE_LISTENER
export LISTENER_NAME=${ORACLE_LISTENER_NAME[@]}
export LISTENER_PASS=${ORACLE_LISTENER_PASS[@]}
export LISTENER_RESTART=${ORACLE_LISTENER_RESTART[@]}
export PFILE=${ORACLE_PFILE}
export MONITOR_PROCESSES=${ORACLE_MONITOR_PROCESSES[@]}
export MONITOR_INTERVAL=$ORACLE_MONITOR_INTERVAL
export TIME_OUT=$ORACLE_TIME_OUT
export PARENT_ENVIRONMENT=$ORACLE_PARENT_ENVIRONMENT
export CLEANUP_BEFORE_STARTUP=$ORACLE_CLEANUP_BEFORE_STARTUP
export USER_SHUTDOWN_MODE=$ORACLE_USER_SHUTDOWN_MODE
export DB_SERVICE=${ORACLE_DB_SERVICE[@]}
export DISABLE_BACKUP_MODE=$ORACLE_DISABLE_BACKUP_MODE

export ASM=$ORACLE_ASM
export ASM_DISKGROUP=${ORACLE_ASM_DISKGROUP[@]}
export ASM_VOLUME_GROUP=${ORACLE_ASM_VOLUME_GROUP[@]}
export ASM_PV=${ORACLE_ASM_PV[@]}
export ASM_HOME=$ORACLE_ASM_HOME
export ASM_USER=$ORACLE_ASM_USER
export ASM_SID=$ORACLE_ASM_SID
export KILL_ASM_FOREGROUNDS=$ORACLE_KILL_ASM_FOREGROUNDS
export RETRY_INTERVAL_ORA_ERROR=$ORACLE_RETRY_INTERVAL_ORA_ERROR
export CRITICAL_PDB=${ORACLE_CRITICAL_PDB[@]}

typeset IS_DATA_GUARD
typeset SGODG="tkit/dataguard/dataguard"

sg_check_module_in_pkg "$SGODG"
if [[ $? -eq 0 ]]; then
    IS_DATA_GUARD="yes"
else
    IS_DATA_GUARD="no"
fi

export IS_DATA_GUARD

if [[ "${IS_DATA_GUARD}" = "yes" ]]; then
    export DC1_ODG_PKG_NAME=$DATAGUARD_DC1_ODG_PKG_NAME
    export DC2_ODG_PKG_NAME=$DATAGUARD_DC2_ODG_PKG_NAME
    export DC1_NODE_LIST=$DATAGUARD_DC1_NODE_LIST
    export DC2_NODE_LIST=$DATAGUARD_DC2_NODE_LIST
fi


read -a MONITOR_PROCESSES <<<$MONITOR_PROCESSES
read -a ASM_DISKGROUP <<<$ASM_DISKGROUP
read -a ASM_VOLUME_GROUP <<<$ASM_VOLUME_GROUP
read -a LISTENER_NAME <<<$LISTENER_NAME
read -a LISTENER_PASS <<<$LISTENER_PASS
read -a LISTENER_RESTART <<<$LISTENER_RESTART
read -a DB_SERVICE <<<$DB_SERVICE
read -a ASM_PV <<<$ASM_PV
read -a CRITICAL_PDB <<<$CRITICAL_PDB

#In Oracle module version .1, the MONITOR_PROCESS variable was  
#not properly populated due to invalid local environment variable
# - $ORACLE_HOME. This is fixed in version .2. Following check
#ensures that if module version .1 is used then the process name
#is proper

index=0
for module in  "${SG_MODULE_NAME[@]}"
do
  if [[ "$module"  = "tkit/oracle/oracle" ]] ; then
   mod_version=${SG_MODULE_VERSION[$index]}
   break
  fi
  (( index = $index + 1 ))
done

if [[ "$mod_version" = "1" ]] ; then
   index=0
   for process in "${MONITOR_PROCESSES[@]}"
   do
       MONITOR_PROCESSES[$index]="${process}${SID_NAME}"
       (( index = $index + 1 ))
   done
fi

if [[ "$INSTANCE_TYPE" = "ASM" ]]; then
   tmp_pfile=${ASM_HOME}/dbs/init${ASM_SID}.ora
else
   tmp_pfile=${ORACLE_HOME}/dbs/init${SID_NAME}.ora
fi

# Confirm that PFILE variable 
if [[ -z "$PFILE" || ! -e "$PFILE" ]] ; then
        PFILE=${tmp_pfile}
fi

op_code=$1
param2=$2
param3=$3
case $op_code in 
   start)
         start_function
         ;;
   stop)
         stop_function
         ;;
   validate)
         validate_function
         ;;
   oracle_monitor)
         monitor_function
         ;;
   oracle_monitor_listener)
         monitor_listener_function $param2
         ;;
   oracle_hang_monitor)
        monitor_database_hang $param2 $param3
        ;;
   oracle_pdb_monitor)
        monitor_pdb
        ;;
   reconfigure_stop)
        reconfigure_stop_function
        ;;
   reconfigure_start)
        reconfigure_start_function
        ;;
    *) 
         sg_log 0 "Usage: ${0} [ start | stop | validate | oracle_monitor | oracle_monitor_listener | oracle_hang_monitor | reconfigure_stop | reconfigure_start ]"
         ;;
esac

exit $exit_val


