#!/bin/bash  
# (c) Copyright 2007-2016 Hewlett Packard Enterprise Development, L.P.
# Version A.12.20.00
###############################################################################
# HA Oracle toolkit Main script
###############################################################################
# Oracle instance can be a database instance or an ASM instance. Based on the
# INSTANCE_TYPE attribute in a package configuration file, this script works on
# the database instance or the ASM instance.
# Arguments
# 1. start: Starts the instance.
# 2. stop: Stops the instance using abort option.
# 3. monitor: Monitors the instance.
# 4. shutdown: Stops the instance using immediate option.
# 5. monitor_listener: Monitors the started listeners.
#
# REVISION HISTORY:
#     Version       mm/dd/yyyy   Changes
#     A.03.00       07/11/2006   1) Added code to handle new argument
#                                   "monitor_listener" for monitoring listener
#                                2) Added code to check for the existence of SPFile
#                                   if PFile is not present.
#     A.04.00                    1) Made UPCC compliant
#                                2) Added code to check whether parent environment is
#                                   needed when a switch to user Oracle is made.
#                                3) Removed code for Oracle version check
#     A.05.00       09/29/2008   1) Made changes for ASM support.
#     A.05.01       04/30/2012   1) Made changes for Oracle 11gR ASM support.
#                                2) Added code to support Multiple Listeners.
#                                3) Added code to support User defined database
#                                   services.
#                                4) Added code to support database startup mode.
#     A.05.01.01    10/12/2012   1) Added code to support online package modification.
#                                2) Added code to support Oracle database backup
#                                   mode.
#     A.05.01.02    03/08/2013   1) Replaced the command ps with pgrep
#                                   for better matching results.
#                                2) Corrected the numeric comparison and made
#                                   them consistent.
#     A.05.01.10    04/18/2013   1) Removed a "\n" character from the log message
#                                   coming from oracle_abort_cmds function.
#                                2) Allowing user defined environment variables
#                                   to be sourced.
#     A.12.00.00    09/27/2013   1) Removed Oracle toolkit maintenance mode.
#                                2) Changes to obsolete legacy style of
#                                   packaging.
#     A.12.00.00    11/08/2013   1) Removed redundant PFILE/SPFILE check
#     A.12.00.20    08/14/2014   1) Enhanced Error and Warning messages to support
#                                   Error handling feature of Servicguard Manager.
#     A.12.00.40    10/09/2015   1) Fix for QXCR1001430463
#                                2) Changes to support 12cR1 version Oracle database.
#                                3) Enhanced Error and sanity checking for
#                                   Oracle ASM diskgroup mount and umount commands
#     A.12.00.50    03/03/2016   1) Moved check_oracle_version function to the
#                                   Oracle toolkit utility script
#     A.12.20.00    10/16/2016   1) Added force flag in function diskgroup_cmds
#                                   to be used with mount option to start the
#                                   package, when asm_mirror module is included
#                                   and RPO is set to "-1".
###############################################################################
#

. /etc/cmcluster.conf
#Source the haoracle_utils.sh
ORACLETKIT_UTILS="${SGROOT}/oracletoolkit/haoracle_utils.sh"
if [[ -f  $ORACLETKIT_UTILS ]]; then
    . ${ORACLETKIT_UTILS}
    if [[ $? -ne 0 ]]
    then
        echo "ERROR: Unable to source Oracle toolkit utility functions file: ${ORACLETKIT_UTILS}"
        exit 1
    fi
else
    echo "ERROR: Unable to find Oracle toolkit utility functions file: ${ORACLETKIT_UTILS}"
    exit 1
fi


###############################################################################
# Function check_yes_no
#
# This function echoes the String depending upon the condition variable value
# The condition variable should be either yes or no
# Parameter 1: Name of the Condition Variable
# Parameter 2: Condition variable value ( either yes/no)
# Parameter 3: String to be echoed
# Return 0 if condition 2 is either yes/no otherwise return 1
#
# If parameter 2 is "yes" then parameter 3 is echoed
# If parameter 2 is "no" then parameter 3 is not echoed
###############################################################################
function check_yes_no
{
    local arg;
    local arg1;

    if [[ -z "$2" ]]; then
        arg="invalid"
    else
        arg=`echo $2 | tr "[:upper:]" "[:lower:]" | tr -d "[:space:]"`
	fi

    if [[ "$2" = "$3" ]]; then
        arg1=$arg
    else
        arg1=$3
    fi 

    case $arg in
        yes)
            echo $arg1
            ;;
        no)
            [ "$arg1" = "$arg" ] && echo $arg1
            ;;
        *)
            echo "Invalid option $1=$2. $1 can be set either as $1=\"yes\" or $1=\"no\""
            return 1
            ;;
    esac
    return 0
}

###############################################################################
# Function: oracle_run_cmds
#
# Start the database instance defined in the environment variables ORACLE_HOME
# and SID_NAME.
# Starts the ASM instance defined in the environment variables ASM_HOME and
# ASM_SID.
###############################################################################
function oracle_run_cmds
{
  if [[ "$CLEANUP_BEFORE_STARTUP" = "yes" ]]; then
        log_message ": Issuing 'shutdown abort' to ensure cleanup"
        . $TKIT_SCRIPT_DIR/haoracle_sql.sh cleanup
        log_message ": Cleanup Done"
  fi

# Oracle Startup
    . $TKIT_SCRIPT_DIR/haoracle_sql.sh start
    exit_status=$?

    # Depending on the return value from haoracle_sql.sh, appropriate actions are taken.
    # 0 - If both Oracle instance and Listener starts successfully
    # 1 - If Oracle instance startup failed
    # 2 - If Oracle instance starts Successfully; Listener is not configured to start
    # 3 - If Oracle instance starts Successfully and Listener failed to start

    case $exit_status in
     0)  
        log_message "Oracle instance startup done."
        return 0;
        ;;
     1)
        log_message "Oracle instance startup failed."
        check_return 1 1
        ;;
     2)
        log_message "Oracle instance startup done; Not starting the listener, as not configured to start."
        return 0;
        ;;
     3) 
        log_message "Oracle Listener $LISTENER_NAME start failed. Aborting Database startup."
        check_return 1 1
        ;;
    esac
}


###############################################################################
# Function: oracle_shutdown_cmds
#
# Shutdown the ASM or database instance using the shutdown immediate command.
# This will do a clean shutdown of the database.
# Do NOT use 'cmhaltpkg' to shutdown the instance as 'cmhaltpkg'
# does an instance abort and all transactions in memory will be lost.
###############################################################################
function oracle_shutdown_cmds
{

#Oracle Shutdown using immediate option 
    . $TKIT_SCRIPT_DIR/haoracle_sql.sh immediate
    if [[ $? -ne 0 ]]
    then
        log_message "Oracle instance shutdown failed."
        check_return 1 2
    else
        log_message "Oracle instance shutdown done."
    fi

return 0

}


###############################################################################
# Function: oracle_abort_cmds
#
# Shutdown function of Oracle uses "abort" method. As "immediate" and "normal"
# will wait for all users to log off and hence shutdown time is 
# non-deterministic. By recursively calling this script with the "kill" option, 
# any process that will not die normally after waiting for the TIME_OUT period 
# can be killed.
###############################################################################
function oracle_abort_cmds
{
#   Call this script with the kill option to terminate any process that will
#   not die normally after waiting for the TIME_OUT period.
#   By calling kill and sending the process to background makes the invoking
#   shell proceed with the abort operation. If the abort operation does not
#   complete within TIME_OUT period, the background process terminates all
#   lingering processes.

       set -m                                                             
       . $TKIT_SCRIPT_DIR/haoracle.sh kill &

# Oracle Shutdown using abort option
    . $TKIT_SCRIPT_DIR/haoracle_sql.sh abort

#   Make sure all processes have gone away before determining that shutdown is
#   complete.  This stops the other node from starting up the package before it
#   has been stopped and the file system has been unmounted.

    typeset -i c d
    typeset -i num_procs=${#MONITOR_PROCESSES[@]}
    typeset -i num_lis=${#LISTENER_NAME[@]}
    while true
    do
        for i in ${MONITOR_PROCESSES[@]}
        do
        id=`ps -u ${ORACLE_ADMIN} -o pid,args | awk "\\$2 ~ /^$i$/{print \\$1}"`
        if [[ "${id}" = "" ]]                                     #JAGad06432
            then
                log_message "*** ${i} process is not running. ***"
                c=0
                while [[ $c -lt $num_procs ]]
                do
                    if [[ "${MONITOR_PROCESSES[$c]}" = "$i" ]]
                    then
                        unset MONITOR_PROCESSES[$c]
                        c=$num_procs
                    fi
                    (( c = c + 1 ))
                done
            fi
        done
#   Make sure listener process is not running. Listener process may be left
#   lingering in cases where the configured listener password is invalid.

        lid=""
        if [[ "${INSTANCE_TYPE}" = "database" ]] && [[ "${LISTENER}" = "yes" ]]; then
		for j in ${LISTENER_NAME[@]}
		do
                lid=`ps -u ${ORACLE_ADMIN} -o pid,args | grep "${ORACLE_HOME}/bin/tnslsnr ${j}" | grep -v grep | awk "\\$3 ~ /^${j}$/{print \\$1}"`
                if [[ "${lid}" = "" ]]; then
                        log_message "*** ${ORACLE_HOME}/bin/tnslsnr ${j} process not running ***"
			d=0
                        while [[ $d -lt $num_lis ]]
                        do
                          if [[ "${LISTENER_NAME[$d]}" = "${j}" ]]; then
                                unset LISTENER_NAME
                                d=$num_lis
                          fi
                          (( d = d + 1 ))
                        done
		fi
		done
        fi

        if [[ "${MONITOR_PROCESSES[@]}" = "" ]] && [[ "${LISTENER_NAME[@]}" = "" && "${LISTENER}" = "yes" || "${LISTENER}" = "no" ]]
        then
#           The shutdown was successful; therefore, get rid of the script (which
#           was started earlier) to kill any existing hung processes.  Check to 
#           see if the script is still running.  If the result from the 'job'
#           command indicates that the script is done, then there is no need 
#           to kill it.

            job=$(jobs | grep -v Done)
            if [[ "${job}" != "" ]]
            then
                log_message "Killing the kill_hung_processes script"
                kill %1 > /dev/null 2>&1
            fi
            return 0                                          # fix for 1653287920
        fi

        sleep 5
     done
}


###############################################################################
# Function: check_process
#
# This process takes one of three arguments: bg, fg or killfg
# bg looks for ASM background processes and server foregrounds
# fg looks for non-server foregrounds
# killfg kills non-server foregrounds
#
# returns 0 if processes with file descriptors are found
# returns 1 if processes with file descriptors are not found
###############################################################################

function check_process
{
     typeset -a pid_list
     typeset -i index_dev=0 num_pids=0 index_pid=0 ret
     typeset curr_dev procname

     proc_type=$1
     if [[ "$proc_type" = "bg" ]]; then
          grep_str="-e ^asm_ -e ^ora_"
     elif [[ "$proc_type" = "fg" ]] || [[ "$proc_type" = "killfg" ]]; then
          grep_str="-e ^oracle${ASM_SID}"
     else
	  return 1
     fi

     while [[ $index_dev -lt $num_devs ]]
     do
          curr_dev=${ASM_DEV[$index_dev]}

          unset pid_list
          for pid in `fuser ${curr_dev} 2>/dev/null | xargs `
          do
               echo $pid |  grep -q -e ${curr_dev} 2>/dev/null
               if [[ $? -ne 0 ]]; then
                  pid_list[${#pid_list[@]}]=$pid;
               fi
          done

          num_pids=${#pid_list[*]}
          index_pid=0
          while [[ $index_pid -lt $num_pids ]]
          do
              i=${pid_list[$index_pid]}
              procname=`ps -p $i -o args | tail -1 | awk '{print $1}'`
              echo $procname | grep -q $grep_str
              ret=$?
              if [[ $ret -eq 0 ]]; then
                    if [[ "$proc_type" = "killfg" ]]; then
                          log_message ": Killing process $i $procname"
                          kill -9 $i
                          kill_proc[${#kill_proc[@]}]=$i
                    else
                          return 0
                    fi
              fi
              (( index_pid = index_pid + 1 ))
          done
          (( index_dev = index_dev + 1 ))
     done
     return 1
}

###############################################################################
# Function: wait_for_bg_descriptor_close
#
# This waits for the ASM background and server foreground processes to close
# the file descriptors on the dismounted disk group volumes. Then checks
# every 3 seconds until FD_TIMEOUT. Returns 1 if the file descriptor close
# takes longer than FD_TIMEOUT. Returns 0 if the file descriptors are closed
# before FD_TIMEOUT and does not wait for the completion of the FD_TIMEOUT.
###############################################################################

function wait_for_bg_descriptor_close
{
    typeset -i count=0 ret
    while [[ $count -lt $FD_TIMEOUT ]]
    do
          check_process bg
          ret=$?
          if [[ $ret -eq 0 ]]; then
                sleep 3
                (( count = count + 3 ))
                continue
          elif [[ $ret -eq 1 ]]; then
                  log_message ": ASM Backgrounds have closed file descriptors"
                  return 0
          fi
    done
    return 1
}

###############################################################################
# Function: diskgroup_cmds
#
# This will mount or dismount the ASM disk groups associated with a database
# instance based on the option passed to this function. The option can be
# either "mount" or "dismount".
# If there are any error encountered during mount or dismount operations,
# messages will be logged to the package log file
###############################################################################

function diskgroup_cmds
{
     typeset -i flag_error=0
     typeset force_flag=""
     typeset ORA_CMD_LOG_FILE="${ORACLE_ORACLE_HOME}/log/.ora_cmd_log_file_${ORACLE_SID_NAME}"
     typeset DG_MNT_ERR_FILE="${ORACLE_ORACLE_HOME}/log/.dg_mnt_err_file_${ORACLE_SID_NAME}"
     typeset cmviewcl_output=`$cmviewcl -vf line -s config 2>/dev/null`
     echo "$cmviewcl_output" |\
          grep -w "node:$HOST|virt_tech=vmware" >> /dev/null 2>&1
     if [[ "$1" == "mount" ]]; then
         if [[ $? -eq 0 ]]; then
             # If there are VMFS files specified, then only issue Oracle ASM 
             # rescan disks
             sg_check_module_in_pkg $VMFS_MODULE
             if [[ $? -eq 0 ]]; then
                 num_vmdk=${#SG_VMDK_FILE_NAME[*]}
                 if [[ $num_vmdk -ge 1 ]]
                 then
                     # In VMware DLS configuration Raw disks will be presented 
                     # to VM at startup to recreate ASM disks, issue Re-scan
                     # on Oracle ASM disks. 
                     log_message "Cleaning any stale ASM disks and instantiating the ASM disks again on $HOST"
                     $ORACLEASM scandisks >/dev/null 2>&1
                     if [[ $? -eq 0 ]];then
                         log_message "Successfully re-instantiated Oracle ASM disks on $HOST"
                     else  
                         log_message "WARNING: Unable to re-instantiate Oracle ASM disks on $HOST"
                     fi 
                 fi
             fi
         fi
         # If asm_mirror module exists and RPO parameter specified as 
         # "-1" (ignore) and PR/VMFS module detected some warning 
         # then only try to mount diskgroup forcefully
         # using "force" option.
         sg_check_module_in_pkg $ASM_MIRROR_MODULE
         if [[ $? -eq 0 ]]; then
             if [[ "$ORA_ASM_MIRROR_RPO_TARGET" == "-1" && 
                   "$SFC_ASM_MIRROR_FAILURE" == "1" ]];then
                 force_flag="force"
             fi 
         fi 
     fi 
     if [[ -f ${ORA_CMD_LOG_FILE} ]]; then
         rm -f ${ORA_CMD_LOG_FILE}
     fi
     $CMD ${ASM_USER} -c "/bin/bash" <<EOF >> ${ORA_CMD_LOG_FILE}
     if [[ "$ORACLE_VERSION" = "11gR2" || "$ORACLE_VERSION" = "12cR1" ]]; then
         export ORACLE_HOME=${ASM_HOME}
     else
         export ORACLE_HOME=${ORACLE_HOME}
     fi

     export ORACLE_SID=${ASM_SID}
     export option=$1
     for i in ${ASM_DISKGROUP[@]}
     do
        #check if diskgroup is already mounted during mount operation. If it is already mounted then we should not try
        #to remount it again
       if [[ "$ORACLE_VERSION" = "11gR1" ]] || [[ "$ORACLE_VERSION" = "11gR2" ]] || [[ "$ORACLE_VERSION" = "12cR1" ]]; then
          echo "select concat('dg_state:',state)A from v\\\$asm_diskgroup where name = '\$i';" | ${ORACLE_HOME}/bin/sqlplus '/ as sysasm' | grep "dg_state:MOUNTED" > /dev/null
          result=\$?
       else
          echo "select concat('dg_state:',state)A from v\\\$asm_diskgroup where name = '\$i';" | ${ORACLE_HOME}/bin/sqlplus '/ as sysdba'  | grep "dg_state:MOUNTED" > /dev/null
          result=\$?
       fi
        if [[ "\$option" = "mount" && \$result -eq 0 ]]; then
           continue
        else
   	   if [[ "$ORACLE_VERSION" = "11gR1" ]] || [[ "$ORACLE_VERSION" = "11gR2" ]] || [[ "$ORACLE_VERSION" = "12cR1" ]]
	   then
                if [[ -z "\$force_flag" ]];then 
	 	    echo "alter diskgroup \${i} \${option};" | ${ORACLE_HOME}/bin/sqlplus '/ as sysasm' | grep "ORA-"
                    ret=\$?
                else
                    echo "alter diskgroup \${i} \${option} ${force_flag};" | ${ORACLE_HOME}/bin/sqlplus '/ as sysasm'>"${ORA_TKIT_LOG_FILE}" 2>&1
                    `cat "${ORA_TKIT_LOG_FILE}" 2>/dev/null | grep -q -w "Diskgroup altered."` >/dev/null 2>&1
                    exit_code=\$?
                    if [[ \$exit_code -ne 0 ]];then
                        `cat "${ORA_TKIT_LOG_FILE}" 2>/dev/null | grep -q "ORA-15230"` >/dev/null 2>&1
                        exit_code=\$?
                        if [[ \$exit_code -eq 0 ]];then
                            echo "alter diskgroup \${i} \${option};" | ${ORACLE_HOME}/bin/sqlplus '/ as sysasm' | grep "ORA-"
                            ret=\$?
                        else
                            ret=1
                        fi
                    else
                        ret=1
                    fi
                    #Clean up temp oracle toolkit log file
                    rm -rf ${ORA_TKIT_LOG_FILE}
                fi
	   else
	 	echo "alter diskgroup \${i} \${option};" | ${ORACLE_HOME}/bin/sqlplus '/ as sysdba' | grep "ORA-"
 		ret=\$?
	   fi
           if [[ \$ret -eq 0 ]]; then
              flag_error=1
              echo "ERROR: Failed to \${option} the diskgroup \${i}"
           fi
        fi
     done
     exit \${flag_error};
EOF
     retval=$?
     if [[ $retval -ne 0 ]]; then
         #These are specific error codes to check mount failure if DG
         #name is incorrect. This is needed as ORA-15001 will be sent
         #incorrect DG is dismounted and we should not return 2 then.
         err_count=`cat ${ORA_CMD_LOG_FILE} | grep -E 'ORA-15017|ORA-15040' | wc -l`
         if [[ $err_count -eq 2 ]]; then
             touch ${DG_MNT_ERR_FILE}
         else
             #If disk is not available. DG mount will fail with this
             #error code. If disk mount failed with the above error
             #codes then return 1, i.e. fail package during halt. 
             #Else return 2, i.e. failover to adoptive node.
             cat ${ORA_CMD_LOG_FILE} | grep -qw "ORA-15001"
             if [[ $? -eq 0 ]]; then
                 if [[ -f ${DG_MNT_ERR_FILE} ]]; then
                     rm -f ${DG_MNT_ERR_FILE}
                 else
                     retval=2
                 fi
             fi
         fi
     fi
     return $retval
}

##############################################################################
# Function: diskgroup_mount_cmds
#
# This will call the function diskgroup_cmds with "mount" option
##############################################################################

function diskgroup_mount_cmds
{
    diskgroup_cmds mount
    if [[ $? -ne 0 ]]; then
       return 1
    fi
}

##############################################################################
# Function: diskgroup_dismount_cmds
#
# This will dismount the ASM disk groups associated with a database instance.
# Then checks to make sure that no ASM background and server foreground
# processes have file descriptors open on the dismounted diskgroup volumes.
# Checks if there are any ASM foregrounds of the form oracle<ASM_SID> having
# file descriptors open every 3 seconds for a maximum timeout of 12 seconds.
# If the ASM foregrounds close file descriptors within 12 seconds, the
# function returns 0. Else after 12 seconds, ASM foregrounds of the form
# oracle<ASM_SID> are killed if KILL_ASM_FOREGROUNDS is set to yes and waits
# for the foregrounds to go away.
##############################################################################

function diskgroup_dismount_cmds
{
    typeset -a pids kill_proc
    typeset -i fg_count=0 timeout flag
    typeset -a ASM_DEV
    typeset -a PVS=(${ASM_PV[@]})
    typeset -a VGS=(${ASM_VOLUME_GROUP[@]})

    diskgroup_cmds dismount
    retval=$?
    if [[ $retval -ne 0 ]]; then
        return $retval
    fi

    for pv in ${PVS[@]}
    do
        ASM_DEV[${#ASM_DEV[@]}]=$pv
    done

    for vg in ${VGS[@]}
    do
         for lv in `vgdisplay -v $vg 2>/dev/null | grep 'LV Name' | awk '{ print $NF }'`
         do
            ASM_DEV[${#ASM_DEV[@]}]=$lv
         done
    done
 
    typeset -i num_devs=${#ASM_DEV[*]}

    wait_for_bg_descriptor_close
    if [[ $? -eq 1 ]]; then
         log_message ": Background processes have not closed file descriptors"
         return 1
    fi

    while [[ $fg_count -lt $FG_TIMEOUT ]]
    do
        check_process fg
        ret=$?
        if [[ $ret -eq 0 ]]; then
              sleep 3
              (( fg_count = fg_count + 3 ))
              continue
	elif [[ $ret -eq 1 ]]; then
              log_message ": ASM Foreground processes have closed file descriptors"
              return 0
        fi
    done

    if [[ "$KILL_ASM_FOREGROUNDS" = "yes" ]]; then
        log_message ": Killing ASM foregrounds if file descriptors are open"
        check_process killfg
        timeout=300
        while [[ $timeout -gt 0 ]]
        do
              unset pids
              for dev in ${ASM_DEV[@]}
              do
                   for pid in ` fuser ${dev} 2>/dev/null | xargs`
                   do
                       echo $pid |  grep -q -e ${dev} 2>/dev/null
                       if [[ $? -ne 0 ]]; then
                          pids[${#pids[@]}]=$pid
		       fi              
                   done
              done
              flag=0
              for pid in ${pids[@]}
              do
                   for kill_pid in ${kill_proc[@]}
                   do
                       if [[ ${pid} -eq ${kill_pid} ]]; then
                            sleep 3
                            (( timeout = $timeout - 3 ))
                            flag=1
                            break
                       fi
                   done
                   if [[ $flag -eq 1 ]]; then
                       break
                   fi
              done
              if [[ $flag -eq 0 ]]; then
                   log_message ": ASM Foreground processes have closed file descriptors"
                   return 0
              fi
              if [[ $timeout -le 0 ]]; then
                   log_message ": ASM Foreground processes have not closed file descriptors"
		   return 0
              fi
        done
    else
        log_message ": Not killing ASM foregrounds"
	return 0	
    fi
}


###############################################################################
# Function: kill_hung_processes
#
# If the instance is hung, spawn a background process that will issue a
# a SIGKILL to each of the monitored processes. This allows us to guarantee
# a time limit for clean halt attempts.
###############################################################################

function kill_hung_processes
{
#   Wait for the TIME_OUT period before sending a kill signal.  The TIME_OUT
#   value should be less than the HP Serviceguard package time-out.

    sleep ${TIME_OUT}

    for i in ${MONITOR_PROCESSES[@]}
    do
    id=`ps -u ${ORACLE_ADMIN} -o pid,args | awk "\\$2 ~ /^$i$/{print \\$1}"`
        if [[ "${id}" != "" ]]
        then
            kill -9 ${id}
            check_status=$?
            if [[ $check_status -ne 0 ]]
            then
                log_message "\n *** ${0} kill_hung_processes function did not find process ***\n *** ${i} running. ***\n"
            else
                log_message "\n *** ${0} kill_hung_processes function did find process ***\n *** ${i} running.  Sent SIGKILL. ***\n" 
            fi
        else
            log_message " *** kill_hung_processes function did not find any process running. ***\n"
        fi
    done
#
#  Kill the lingering listener process
#
   if [[ "${INSTANCE_TYPE}" = "database" ]] && [[ "${LISTENER}" = "yes" ]]
   then
       for j in ${LISTENER_NAME[@]}
       do
        lid=`ps -u ${ORACLE_ADMIN} -o pid,args | grep "${ORACLE_HOME}/bin/tnslsnr ${j}" | grep -v grep | awk "\\$3 ~ /^${j}$/{print \\$1}"`
       if [[ "${lid}" != "" ]]
       then
           kill -9 ${lid}
           check_status=$?
           if [[ $check_status -ne 0 ]]
           then
               log_message "\n *** ${0} kill_hung_processes function did not find Listener process ***\n *** ${ORACLE_HOME}/bin/tnslsnr ${j} running. ***\n"
           else
               log_message "\n *** ${0} kill_hung_processes function did find listener process ***\n ${ORACLE_HOME}/bin/tnslsnr ${j} *** running.  Sent SIGKILL. ***\n"
           fi
       fi
       done
   fi

}

###############################################################################
# Function: log_message
#
# This function log_messagees any message with date, time and node name affixed
# to it. It accepts just one paarmeter.
# Parameter: 
# 1. Message to be logged
###############################################################################

function log_message
{
   sg_log 0 "$1"
}

###############################################################################
# Function: check_return
# Check return value of functions and exit. It accepts two parameters.
# Parameters:
# 1. Exit value of the previously executed command
# 2. The place of execution 
#        1 - oracle_run_cmds
#        2 - oracle_abort_cmds
#        3 - Monitor script 
#        4 - Improper parameters to toolkit script
###############################################################################

function check_return
{
    if [[ $1 -ne 0 ]]
    then
    case $2 in
        1)
             log_message "ERROR: Function oracle_run_cmds: Failed to start Oracle instance"
             oracle_abort_cmds
             exit 1
        ;;

        2)
             log_message "ERROR: Function oracle_shutdown_cmds: Failed to stop Oracle instance"
                    exit_code=1
        ;;

        3)   log_message "ERROR: Could not Start the Monitor Function"
             exit 1
        ;;

        4)   log_message "ERROR: Improper parameters in Toolkit Configuration file"
             exit 1
        ;;

	5)   log_message "ERROR: Could not Start the listener Monitor Function"
             exit 1
        ;;

        *)
             log_message "ERROR: Failed, unknown error."
             ;;
    esac
    fi
}

function verify_asm_parameters
{

# Confirm that ASM_HOME variable contains a valid directory name
if [[ ! -d "$ASM_HOME" ]] ; then
    log_message "ERROR: ASM Home $ASM_HOME does not exist"
    check_return 1 4
fi

PATH=$PATH:$ASM_HOME/bin:

}

###############################################################################
# MAIN
# Check the command-line option and take the appropriate action.
###############################################################################
PATH=/bin:/sbin:/usr/bin:/usr/sbin:/usr/lbin
export HOST=`/bin/hostname | cut -d. -f 1`
DATE=`/bin/date`
CAT=/bin/cat

# Servicegard command variables
cmviewcl=${SGSBIN}/cmviewcl

# Oracle ASM related commands
typeset ORACLEASM="/usr/sbin/oracleasm"

# Module defination
typeset VMFS_MODULE="sg/vmfs"
typeset ASM_MIRROR_MODULE="tkit/oracle/asm_mirror"

# FD_TIMEOUT indicates the maximum time in seconds to wait for the ASM background
# processes and Server foreground processes to close the file descriptors on the
# dismounted disk group volumes associated with the database instance.

FD_TIMEOUT=30

# FG_TIMEOUT indicates the maximum time in seconds to wait for the ASM foreground
# processes of the form oracle<ASM_SID> to close the file descriptors on the
# dismounted disk group volumes associated with the database instance.

FG_TIMEOUT=12
exit_code=0

# Include customer defined parameters. 
if [[ -f $TKIT_SCRIPT_DIR/${ORACLE_SID_NAME}_customer.conf ]] ; then 
   . $TKIT_SCRIPT_DIR/${ORACLE_SID_NAME}_customer.conf
   log_message "INFO: Included customer Defined parameters from $TKIT_SCRIPT_DIR/${ORACLE_SID_NAME}_customer.conf."
fi

if [[ "$INSTANCE_TYPE" != "database" ]]; then
   if [[ "$INSTANCE_TYPE" != "ASM" ]]; then
      log_message "ERROR: Improper value for INSTANCE_TYPE"
      check_return 1 4
   fi
fi

if [[ "$INSTANCE_TYPE" = "database" ]]; then

# Confirm that ORACLE_HOME variable contains a valid directory name
if [[ ! -d "$ORACLE_HOME" ]] ; then
    log_message "ERROR: Oracle Home $ORACLE_HOME does not exist"
    check_return 1 4
fi


# Confirm that START_MODE variable contains non-null string and a valid option
if [[ -z "$START_MODE" ]] ; then
    START_MODE="open"
else
    if [[ "$START_MODE" != "nomount" ]] && [[ "$START_MODE" != "mount" ]] && [[ "$START_MODE" != "open" ]] ; then
        log_message "ERROR: Improper value for Startup Mode (START_MODE)"
        check_return 1 4
    fi
fi

LISTENER=`check_yes_no LISTENER "$LISTENER" "$LISTENER"`
check_status=$?
if [[ $check_status -ne 0 ]]; then
    log_message "$LISTENER"
    check_return 1 4
fi

# Confirm that LISTENER_NAME variable contains non-null string if LISTENER=yes
if [ "$LISTENER" = "yes" -a -z "$LISTENER_NAME" ] ; then
    log_message "ERROR: Listener Name (LISTENER_NAME) is not defined in package configuration file."
    check_return 1 4
fi

ASM=`check_yes_no ASM "$ASM" "$ASM"`
check_status=$?
if [[ $check_status -ne 0 ]]; then
     log_message "$ASM"
     check_return 1 4
fi

if [[ "$ASM" = "yes" ]]; then
     verify_asm_parameters
     KILL_ASM_FOREGROUNDS=`check_yes_no KILL_ASM_FOREGROUNDS "$KILL_ASM_FOREGROUNDS" "$KILL_ASM_FOREGROUNDS"`
     check_status=$?
     if [[ $check_status -ne 0 ]]; then
          log_message "$KILL_ASM_FOREGROUNDS"
          check_return 1 4
     fi
fi

elif [[ "$INSTANCE_TYPE" = "ASM" ]]; then
       verify_asm_parameters
       ORACLE_HOME=$ASM_HOME
       ORACLE_ADMIN=$ASM_USER
       SID_NAME=$ASM_SID
fi

export ORACLE_VERSION=`check_oracle_version`


# Setting MONITOR_INTERVAL default value if not defined
if [[ -z "$MONITOR_INTERVAL" ]] ; then
    MONITOR_INTERVAL=30
fi

# Setting TIME_OUT default value if not defined
if [[ -z "$TIME_OUT" ]] ; then
    TIME_OUT=30
fi

# Setting RETRY_INTERVAL_ORA_ERROR default value if not defined
if [[ -z "$RETRY_INTERVAL_ORA_ERROR" ]] ; then
    RETRY_INTERVAL_ORA_ERROR=0
fi

PARENT_ENVIRONMENT=`check_yes_no PARENT_ENVIRONMENT "$PARENT_ENVIRONMENT" "$PARENT_ENVIRONMENT"`
check_status=$?
if [[ $check_status -ne 0 ]]; then
    log_message "$PARENT_ENVIRONMENT"
    check_return 1 4
fi

if [[ "$PARENT_ENVIRONMENT" = "yes" ]]; then
     export CMD="su"
else
     export CMD="su -" 
fi

# Verify that USER_SHUTDOWN_MODE contains a valid value
if [[ "$USER_SHUTDOWN_MODE" != "immediate" ]]; then
     if [[ "$USER_SHUTDOWN_MODE" != "abort" ]]; then
          log_message "ERROR: Invalid value for Shutdown Mode (USER_SHUTDOWN_MODE)"
          check_return 1 4
     fi
fi

CLEANUP_BEFORE_STARTUP=`check_yes_no CLEANUP_BEFORE_STARTUP "$CLEANUP_BEFORE_STARTUP" "$CLEANUP_BEFORE_STARTUP"`
check_status=$?
if [[ $check_status -ne 0 ]]; then
    log_message "$CLEANUP_BEFORE_STARTUP"
    check_return 1 4
fi

if [[ -n "${CRITICAL_PDB[*]}" ]]; then
  j=0
  for var in ${CRITICAL_PDB[@]} 
  do
      CRITICAL_PDB[$j]=`echo $var|tr '[[:upper:]]' '[[:lower:]]'`
      ((j=j+1))
  done
  if [[ "$START_MODE" = "open" ]]; then
     CDB=$(determine_db_type)
     if [[ "$CDB" = "YES" && "${CRITICAL_PDB}" != "none" && "${CRITICAL_PDB}" != "any" ]]; then
         for i in ${CRITICAL_PDB[@]}
         do
            check_if_pdb_exists $i
            if [[ $? -eq 1 ]]; then
                sg_log 0 "ERROR: Specified Critical PDB $i does not exists in the Container Database."
            fi
          done
      fi
   fi
fi

Param2=$2
Param3=$3

# If the DB_SERVICE attribute is set to all, then obtain the list of
# services.
PS=/bin/ps
GREP=/bin/grep
ECHO=/bin/echo
OCSSD="$ASM_HOME/bin/ocssd.bin"
PGREP=/usr/bin/pgrep

$PGREP -f "$OCSSD" > /dev/null
if [[ $? -eq 0 ]]
then
    serv_all=""
    if [[  X"$DB_SERVICE" != "X" ]]
    then
        temp_service=`$ECHO ${DB_SERVICE[0]}|/usr/bin/tr '[:upper:]' '[:lower:]'`
        if [[ X"$temp_service" = "Xall" ]]
        then
            SRVCTL=$ORACLE_HOME/bin/srvctl
            serv_all=`su ${ORACLE_ADMIN} -c '/bin/bash' <<EOF
            export ORACLE_HOME=$ORACLE_HOME
            export SRVCTL=$ORACLE_HOME/bin/srvctl
            export SID_NAME=$SID_NAME
            $SRVCTL config service -d $SID_NAME |$GREP 'Service name'|/usr/bin/cut -d ':' -f 2
EOF`
            if [ $? -ne 0 -o -z "$serv_all" ]
            then
                serv_all=`su ${ORACLE_ADMIN} -c '/bin/bash' <<EOF
                export ORACLE_HOME=$ORACLE_HOME
                export SRVCTL=$ORACLE_HOME/bin/srvctl
                export SID_NAME=$SID_NAME
                $SRVCTL config service -d $SID_NAME |/usr/bin/cut -d ' ' -f 1
EOF`
            fi
            count=0
            for service in ${serv_all[@]}
            do
              DB_SERVICE[$count]=$service
              (( count = $count + 1 ))
            done
        fi
    fi
fi

log_message "*** $0 called with $1 argument. ***"
export ARG=$2
typeset ORA_TKIT_LOG_FILE="${ORACLE_ORACLE_HOME}/log/.ora_tkit_log_file_${ORACLE_SID_NAME}"
case $1 in 

    kill)
        kill_hung_processes
    ;;

    monitor)
        . $TKIT_SCRIPT_DIR/haoracle.mon
        check_status=$?
        check_return $check_status 3
    ;;
    
    monitor_listener)
        . $TKIT_SCRIPT_DIR/halistener.mon $ARG
        check_status=$?
        check_return $check_status 5
    ;;
    hang_monitor)
        if [[ "$Param2" = "" ]]; then
                Param2=30;
        fi
        if [[ "$Param3" = "" ]]; then
                Param3=failure
        fi
        . $TKIT_SCRIPT_DIR/hadbhang.mon $Param2 $Param3
        check_status=$?
        check_return $check_status 5
    ;;
    monitor_pdb) 
        . $TKIT_SCRIPT_DIR/haoraclepdb.mon
        check_status=$? 
        check_return $check_status 5
    ;;

    start)
        if [[ "$INSTANCE_TYPE" = "database" && "$ASM" = "yes" ]]; then
              log_message ": Mounting diskgroups"
              diskgroup_mount_cmds
              if [[ $? -eq 1 ]]; then
                 log_message "ERROR: One or more diskgroups have failed to mount"
                 exit 1
              fi 
        fi
        log_message ": Starting Oracle SESSION $SID_NAME at ${DATE}"
        oracle_run_cmds
    ;;

    stop)
        log_message ": Aborting Oracle SESSION $SID_NAME at ${DATE}"
        if [[ $# -eq 2 ]] && [[ "$2" = "user" ]] && [[ "$USER_SHUTDOWN_MODE" = "immediate" ]]; then
              log_message ": Invoking shutdown immediate of Oracle SESSION $SID_NAME at ${DATE}"
              oracle_shutdown_cmds
        else
              log_message ": Invoking shutdown abort of Oracle SESSION $SID_NAME at ${DATE}"
              oracle_abort_cmds
        fi
        if [[ "$INSTANCE_TYPE" = "database" && "$ASM" = "yes" ]]; then
              log_message ": Dismounting diskgroups"
              diskgroup_dismount_cmds
              retval=$?
              if [[ $retval -ne 0 ]]; then
                    log_message "INFO: Background processes still have file descriptors open"
                    log_message "ERROR: One or more diskgroup have failed to dismount"
                    exit $retval
              fi
        fi
    ;;

    shutdown)
        log_message ": Shutting down Oracle SESSION $SID_NAME at ${DATE}"
        oracle_shutdown_cmds
        if [[ "$INSTANCE_TYPE" = "database" && "$ASM" = "yes" ]]; then
              log_message ": Dismounting diskgroups"
              diskgroup_dismount_cmds
              if [[ $? -eq 1 ]]; then
                    log_message ": Background processes still have file descriptors open"
              fi
        fi
    ;;

    *)
        log_message "Usage: ${0} [ start | stop | monitor | shutdown | monitor_listener | hang_monitor ]"
    ;;
esac
exit $exit_code
