#!/bin/bash
# (c) Copyright 2016-2017 Hewlett Packard Enterprise Development LP.
# (c) Copyright 2015 Hewlett Packard Enterprise Development LP.
# Version A.12.10.00
###############################################################################
# HA Oracle Startup/Shutdown script
###############################################################################
# This script accepts one argument
# start/abort/immediate : If start is passed the oracle instance is started.
#                         If abort is passed  oracle instance is
#                         brought down with "shutdown abort" SQL command.
#                         If immediate is passed oracle instance is
#                         brought down with "shutdown immediate" SQL command.
#
# REVISION HISTORY:
#     Version      mm/dd/yyyy   Changes
#     A.03.00      07/11/2006   1) Modified code to start Oracle instance using 
#		        	   SPFile if PFile is not present
#			        2) Added -s "/bin/bash" to su command	
#			        3) Modified code to log all Oracle messages in
#			           the package log file
#     A.03.01      11/30/2006   1) Modified code to start listerner first
#     A.04.00                   2) Added code to handle parent environment
#                                  when a switch to user Oracle is made 
#     A.05.00      09/29/2008   1) Made changes for ASM support.
#     A.05.01      04/30/2012   1) Made changes for Oracle 11gR ASM support.
#                               2) Added code to support Multiple Listeners.
#                               3) Added code to support User defined database
#                                  services.
#                               4) Added code to support database startup mode.
#     A.05.01.01   12/10/2012   1) Added code to support Alert mail notification.
#                               2) Added code to support oracle database backup
#                                  mode.
#     A.05.01.02   03/08/2013   1) Replaced the command ps with pgrep
#                                  for better matching results.
#     A.05.01.10   04/18/2013   1) Changes done to correct string comparisons
#                                  consistent.
#     A.05.01.11   07/22/2013   1) Fixed NLS_LANG issue: changed SQL select
#                                  clause by adding WHERE INSTANCE_NAME='$SID'  
#                               2) When AUTOSTART_CMCLD=1, fixed return code 
#                                  check for pgrep command, added email alert
#                                  and few messages before sleep.  
#     A.12.00.20   08/14/2014   1) Enhanced Error and Warning messages to support
#                                  Error handling feature of Servicguard Manager.
#     A.12.00.40   09/10/2015   1) Enhanced to support starting/stopping PDB's in 
#                                  in the context of Oracle 12c Container databases.
###############################################################################

###############################################################################
# Function:  start_oracle
# This function starts the Oracle instance and Listeners.
# start_oracle returns the following values depending on the starup results.
# Returns  0 - If both Oracle instance and Listener(s) starts successfully
# Returns  1 - If Oracle instance startup failed
# Returns  2 - If Oracle instance starts Successfully;Listener is not configured to start
# Returns  3 - If Oracle instance starts Successfully and Listener(s) failed to start
###############################################################################
function start_oracle
{

# For 11GR2 and later versions of Oracle, if the AUTOSTART_CMCLD is set to 1 and has an ASM instance,
# verify if the Oracle Clusterware is running before starting the package. If Oracle Clusterware
# is not running, wait for it to start. If the Oracle Clusterware does not start within the expected
# time, log an error message in the package log file and exit.

if [[ -f /etc/redhat-release ]]; then 
AUTOSTART="/usr/local/cmcluster/conf/cmcluster.rc"
else
AUTOSTART="/opt/cmcluster/conf/cmcluster.rc"
fi

if [[ -n "$PEV_ATX_ORACLE_LSNR" ]]; then
    HPE_ATX_LISTENER_CMD="/usr/bin/hpe-atx"
    atx_listener_log_flag=''
    PEV_ATX_LISTENER_Array=($PEV_ATX_ORACLE_LSNR)
    for atx_parm_val in "${!PEV_ATX_LISTENER_Array[@]}";
    do
        if [ "${PEV_ATX_LISTENER_Array[$atx_parm_val]}" = "-l" ] || [ "${PEV_ATX_LISTENER_Array[$atx_parm_val]}" = "--log" ]; then
            atx_log_index=${atx_parm_val}
            ((atx_log_index=atx_log_index+1))
            if [[ "${PEV_ATX_LISTENER_Array[$atx_parm_val]}" = "-l" ]]; then
                atx_listener_log_val="-l ${PEV_ATX_LISTENER_Array[$atx_log_index]}"
            else
                atx_listener_log_val="--log ${PEV_ATX_LISTENER_Array[$atx_log_index]}"
            fi
            PEV_ATX_LISTENER_Array[$atx_parm_val]=''
            PEV_ATX_LISTENER_Array[$atx_log_index]=''
            atx_listener_log_flag='enabled'
        else
            HPE_ATX_LISTENER_CMD+=" ${PEV_ATX_LISTENER_Array[$atx_parm_val]}"
        fi
    done
fi


AUTO_CMCLD=`cat $AUTOSTART | $GREP AUTOSTART_CMCLD= | $AWK -F"=" '{print $2}'`
if [[ "$ORACLE_VERSION" = "11gR2" || "$ORACLE_VERSION" = "12cR1" ]] && [[ "$AUTO_CMCLD" = "1" ]] && [[ "$INSTANCE_TYPE" = "ASM" ]]
then
    typeset -i crs_wait_count=0
    while true
    do
      $PGREP -f "$OCSSD" > /dev/null
      if [[ $? -ne 0 ]]
      then
          if [[ $crs_wait_count -eq 0 ]]
          then
              log_message "Waiting for Oracle Clusterware to start. This might take sometime..."
              log_message "WARNING: Oracle toolkit - If this package restart is not due to either system restart or reboot, start Oracle Clusterware manually."
              sg_send_mail "ASM package $SG_PACKAGE is waiting for Oracle Clusterware to start on node $HOST"
          fi
          $SLEEP 10
          crs_wait_count=$(( $crs_wait_count + 1 ))
          if [[ $crs_wait_count -eq 60 ]]
          then
              log_message "ERROR: Oracle Clusterware is not running. Start Oracle Clusterware before starting the package."
              exit 1
          fi
      else
          if [[ $crs_wait_count -ne 0 ]]
          then
              log_message "Waiting for 40 seconds to have the Oracle Clusterware ready after startup..."
              $SLEEP 40
          else
              log_message "Waiting for 10 seconds to have the ASM instance to be ready..."
              $SLEEP 10
          fi
          break
      fi
    done
fi

typeset dataguard
typeset SGODG="tkit/dataguard/dataguard"
sg_check_module_in_pkg "$SGODG"
if [[ $? -eq 0 ]]; then
    dataguard="yes"
else
    dataguard="no"
fi

#  To start Oracle the following code will be executed after becoming oracle
#  administrator. After starting Oracle it gets the INSTANCE_NAME value from the
#  v$instance table and compares the value with the SID_NAME. If it is matching
#  it proceeds with the Listeners Starting if the user has opted for it.

$CMD ${ORACLE_ADMIN} -s "/bin/bash"  <<EOF

export ORACLE_HOME=${ORACLE_HOME}
export ORACLE_SID=${SID_NAME}

if [[ -e "$PFILE" ]] ; then
	command="pfile=$PFILE";
else
	command=""
fi


if [[ "$INSTANCE_TYPE" = "database" ]] && [[ "$LISTENER" = "yes" ]]; then
	for I in ${LISTENER_NAME[@]}
	do
             #Check if listener needs to be started through ATX
             if [ ! -z "$PEV_ATX_ORACLE_LSNR" ]; then
                 #start through ATX
                 if [ ! -z "$atx_listener_log_flag" ]; then
                     ${HPE_ATX_LISTENER_CMD} ${atx_listener_log_val}_\${I} ${ORACLE_HOME}/bin/lsnrctl start \${I}
                 else
                     ${HPE_ATX_LISTENER_CMD} ${ORACLE_HOME}/bin/lsnrctl start \${I}
                 fi
             else
                 ${ORACLE_HOME}/bin/lsnrctl start \${I}
             fi
             exit_status=\$?
             wait;

	     if [[ \$exit_status -ne 0 ]]
	     then
	         echo "Oracle lsnrctl failed to start \${I}"
	     fi
	done
fi

if [[ "$INSTANCE_TYPE" = "ASM" ]]; then
        # Pulled this line out from the below here-doc to facilitate Oracle logging.
	if [[ "$ORACLE_VERSION" = "11gR1" ]] || [[ "$ORACLE_VERSION" = "11gR2" ]] || [[ "$ORACLE_VERSION" = "12cR1" ]]
	then
		echo "startup nomount \${command};" | ${ORACLE_HOME}/bin/sqlplus '/ as sysasm'
	else
		echo "startup nomount \${command};" | ${ORACLE_HOME}/bin/sqlplus '/ as sysdba'
	fi
else
   if [[ "$START_MODE" = "nomount" ]]; then
      echo "startup nomount \${command};" | ${ORACLE_HOME}/bin/sqlplus '/ as sysdba'
   else
      if [ -n "$DISABLE_BACKUP_MODE" -a "$DISABLE_BACKUP_MODE" = "yes" ]; then
         echo "startup mount \${command};" | ${ORACLE_HOME}/bin/sqlplus '/ as sysdba'
         #Check if database is in active mode. This check is applicable for both non-CDB and CDB databases.
         echo "select status from v\\\$backup;" | ${ORACLE_HOME}/bin/sqlplus '/ as sysdba'| $GREP -q -e "^ACTIVE"
         if [[ \$? -ne 0 ]]; then
            echo "Oracle database is not in the backup mode."
         else
            echo "Oracle database is in the backup mode."
            if [[ "$START_MODE" = "open" ]]; then
			   echo "INFO: Oracle toolkit ending the backup to ensure the successful start of the package."  
               echo "alter database end backup;" | ${ORACLE_HOME}/bin/sqlplus '/ as sysdba' | $GREP "ORA-"
               if [[ \$? -eq 0 ]]; then
                  echo "ERROR: Unable to get the database out of the backup mode. Oracle user must get the database out of the backup mode before starting the package."
                  exit 1;
               fi
            else
               echo "Oracle database is in the backup mode. Oracle user must get the database out of the backup mode before starting the package."
               exit 1;
            fi
         fi
         if [[ "$START_MODE" = "open" ]]; then
            sqltext1="alter database open;"
            query_out=\`echo "\$sqltext1" | ${ORACLE_HOME}/bin/sqlplus '/ as sysdba' | $GREP -e "ORA-" \`
            if [[ -z "\$query_out" ]]; then # This condition is applicable when DB is not open prior to the package startup.
               
               echo "Database is opened."
            else
               echo "\$query_out" | grep -q "ORA-01531"
               if [[ \$? -eq 0 ]]; then
                  echo "Database is already opened."
               else
                  echo "ERROR: Unable to open the database."
                  exit 1;
               fi
            fi 
         fi 
       else
         echo "startup $START_MODE \${command};" | ${ORACLE_HOME}/bin/sqlplus '/ as sysdba'
       fi 
   fi
fi

case $START_MODE in
nomount)
    start_mode="STARTED"
;;
mount)
    start_mode="MOUNTED"
;;
open)
    start_mode="OPEN"
;;
*)
    echo "invalid startup option"
;;
esac

exit_status=1;
if [[ "$INSTANCE_TYPE" = "database" ]]; then
    if [[ "$dataguard" = "yes" ]]; then
        ${ORACLE_HOME}/bin/sqlplus '/ as sysdba' <<SQL | grep -q -i  '\(open\|mounted\)'
        select STATUS from v\\\$instance WHERE INSTANCE_NAME='${SID_NAME}';
        exit \$?
SQL
        if [[ \$? -eq 0 ]]; then
            exit_status=0;
        fi
    else
        if [[ "$dataguard" = "no" ]]; then
            ${ORACLE_HOME}/bin/sqlplus '/ as sysdba' <<SQL  | grep -q -e "\${start_mode}"
            select STATUS from v\\\$instance WHERE INSTANCE_NAME='${SID_NAME}';
            exit \$?;
SQL
            if [[ \$? -eq 0 ]]; then
                exit_status=0;
            fi
        fi
    fi
fi
if [[ "$INSTANCE_TYPE" = "ASM" ]]; then
        ${ORACLE_HOME}/bin/sqlplus '/ as sysdba' <<SQL | grep -q -e "^${SID_NAME}$"
        select INSTANCE_NAME from v\\\$instance;
        exit \$?;
SQL
if [[ \$? -eq 0 ]]; then
exit_status=0;
fi
fi

wait;

if [[ \${exit_status} -ne 0 ]]; then
exit 1;
else
if [[ "$INSTANCE_TYPE" = "database" ]] && [[ "$LISTENER" = "no" ]]; then
exit 2;
fi
exit 0
fi

EOF

oracle_start=$?

#If a CDB database is being packaged, then start all the PDB's
#associated with CDB if the START_MODE is open.
if [[ $oracle_start -ne 1 && "$START_MODE" = "open" ]]; then
   #Check if the DB started is CDB, if yes then start the PDB associated
   #with it if the START_MODE of the CDB is "open"
   CDB=$(determine_db_type)
   if [[ -n "$CDB" && "$CDB" = "YES" ]]; then
      pdb_state_change open
      if [[ $? -eq 0 ]]; then
         log_message "INFO: All Pluggable databases are started successfully"
      else
         log_message "INFO: One or more Pluggable database have failed to start"
      fi
      status=$(db_get_pdb_status)
      if [[ -n "${status}" ]]; then
         log_message "INFO: Status of the Pluggable databases which are not NORMAL are as follows: ${status}"
      fi
   fi
fi

$PGREP -f "$OCSSD" > /dev/null
if [[ $? -eq 0 ]]
then
    for service in ${DB_SERVICE[@]}
    do
    su ${ORACLE_ADMIN} -c '/bin/bash' <<EOF
       cd $ORACLE_HOME
       export ORACLE_HOME=$ORACLE_HOME
       SRVCTL=$ORACLE_HOME/bin/srvctl
       export service=$service
       retry=0
       count=0
       random_delay=0
       service_status=\`\$SRVCTL status service -s $service -d $SID_NAME \`
       if [[ \$? -eq 0 ]]
       then
           $ECHO \$service_status| $GREP  -e  "PRKO-2017"  -e "PRCR-1001"
           if [[ \$? -ne 0 ]]
           then
               $ECHO "Starting $service service..........."
               while [[ \$retry -le \$count ]]
               do
                  result=\`\$SRVCTL start service -s $service -d $SID_NAME\`
                  if [[ \$? -eq 0 ]]
                  then
                      $ECHO "Started $service service successfuly "
                  else

                       # check  for resource busy error
                       # retry (max 3) after random delay to
                       # start the service.

                        $ECHO "\$result" | $GREP -e "CRS-0223" -e "CRS-0233"
                        if [[  \$? -eq 0 ]]
                        then
                            random_delay=`expr '(' $RANDOM '*' 10 / 32767 ')' + 5`
                            $SLEEP \$random_delay # try again after a while
                            count=2
                        else
                            # some other error , lets display the error message
                            $ECHO \$result
                            $ECHO "Could not start $service service "
                        fi
                   fi
                   (( retry = \$retry + 1 ))
               done
           fi
       else
           $ECHO "Could not retrive status of $service service "

       fi

EOF
   done
fi

return $oracle_start

}

##############################################################################
# Function: cleanup_oracle
# This function is responsible for issuing a 'shutdown abort' to cleanup
# uncleared shared memory and semaphores
##############################################################################
function cleanup_oracle
{

# Close all the PDB's in the context of CDB database.
if [[ "$START_MODE" = "open" && "$CDB" = "YES" ]]; then
   pdb_state_change close
   if [[ $? -eq 0 ]]; then
      log_message "INFO: All Pluggable databases are closed successfully"
   else
      log_message "INFO: One or more Pluggable database have failed to stop"
   fi
   status=$(db_get_pdb_status)
   if [[ -n "${status}" ]]; then
      log_message "INFO: Status of the Pluggable databases which are not NORMAL are as follows: ${status}"
   fi
elif [[ "$START_MODE" != "open" || "$CDB" != "YES" ]]; then
   log_message "INFO: Database is not opened. No change to the Pluggable database is done during cleanup."
fi

$CMD ${ORACLE_ADMIN} -c "/bin/bash" <<EOF 2>/dev/null
export ORACLE_HOME=${ORACLE_HOME}
export ORACLE_SID=${SID_NAME}
if [[ "$INSTANCE_TYPE" = "ASM" ]]; then
	if [[ "$ORACLE_VERSION" = "11gR1" ]] || [[ "$ORACLE_VERSION" = "11gR2" ]] || [[ "$ORACLE_VERSION" = "12cR1" ]]
	then
		echo "shutdown abort" | ${ORACLE_HOME}/bin/sqlplus '/ as sysasm';
	else
		echo "shutdown abort" | ${ORACLE_HOME}/bin/sqlplus '/ as sysdba';
	fi
else 
	echo "shutdown abort" | ${ORACLE_HOME}/bin/sqlplus '/ as sysdba';
fi
EOF
}


###############################################################################
# Function:  shutdown_oracle
# This function is responsible for bringing down Oracle instance and Listener.
###############################################################################
function shutdown_oracle
{

option=$1;
$PGREP -f $OCSSD > /dev/null
if [[ $? -eq 0 ]]
then
    for service in ${DB_SERVICE[@]}
    do
    su ${ORACLE_ADMIN} -c '/bin/bash' <<EOF
       cd $ORACLE_HOME
       export ORACLE_HOME=$ORACLE_HOME
       export service=$service
       SRVCTL=$ORACLE_HOME/bin/srvctl
       retry=0
       count=0
       random_delay=0
       service_status=\`\$SRVCTL status service -s $service -d $SID_NAME \`
       if [[ \$? -eq 0 ]]
       then
           $ECHO \$service_status|$GREP -e "PRKO-2017"  -e "PRCR-1001"
           if [[ \$? -ne 0 ]]
           then
               $ECHO \$service_status|$GREP -q 'is not running'
               if [[ \$? -ne 0 ]]
               then
                   $ECHO "Stopping $service service on $SID_NAME..."
                   while [[ \$retry -le \$count ]]
                   do
                       result=\`\$SRVCTL stop service -s $service -d $SID_NAME \`
                       if [[ \$? -eq 0 ]]
                       then

                          # in 10gR2 and 11gR1, srvctl stop service command
                          # return 0 even if the service is already stopped.
                          # if the following grep succeeds then it will show
                          # the error message that the service is already stopped
                          # on the ORACLE_SID instance.

                           $ECHO "\$result" | $GREP "PRKP-1065"
                           if [[ \$? -ne 0 ]]
                           then
                               $ECHO "Stopped $service service successfully"
                           fi
                       else
                           # check  for resource busy error
                           # retry (max 3) after random delay to stop the service
                           # on the particular instance(ORACLE_SID).

                           $ECHO "\$result" | $GREP -q -e "CRS-0223" -e "CRS-0233"
                           if [[  \$? -eq 0 ]]
                           then
                               random_delay=`expr '(' $RANDOM '*' 10 / 32767 ')' + 5`
                               $SLEEP \$random_delay # try again after a while
                               count=2
                           else
                               # some other error, lets display the error message
                               $ECHO \$result
                               $ECHO "Could not stop $service service on $SID_NAME."
                           fi
                       fi
                       (( retry = \$retry + 1 ))
                   done
               else
                    $ECHO "Service $service is already stopped. "
               fi
           fi
       else
           $ECHO "Could not retrive status of $service service."
       fi
EOF
    done
fi
unset DB_SERVICE[@]

if [[ "$SG_HALT_REASON" != "user_halt" ]] ; then
      sg_send_mail "Oracle toolkit package $SG_PACKAGE failed on node $HOST"
fi

#Close and stop all PDB if database packaged is a CDB database.
if [[ "$START_MODE" = "open" && "$CDB" = "YES" ]]; then
   pdb_state_change close
   if [[ $? -eq 0 ]]; then
      log_message "All Pluggable databases are closed successfully"
   else
      log_message "INFO: One or more Pluggable database have failed to stop"
   fi
   status=$(db_get_pdb_status)
   if [[ -n "${status}" ]]; then
      log_message "INFO: Status of the Pluggable databases which are not NORMAL are as follows: ${status}"
   fi
fi

 
#  To stop the Oracle instance the following code will be executed after
#  becoming the Oracle administrator.

$CMD ${ORACLE_ADMIN} -s "/bin/bash" <<EOF1 2>/dev/null

  export ORACLE_HOME=${ORACLE_HOME}
  export ORACLE_SID=${SID_NAME}
  
  if [[ "$INSTANCE_TYPE" = "ASM" ]]; then
	if [[ "$ORACLE_VERSION" = "11gR1" ]] || [[ "$ORACLE_VERSION" = "11gR2" ]] || [[ "$ORACLE_VERSION" = "12cR1" ]]
	then
		echo "shutdown $option" | ${ORACLE_HOME}/bin/sqlplus '/ as sysasm';
	else
		echo "shutdown $option" | ${ORACLE_HOME}/bin/sqlplus '/ as sysdba';
	fi
  else
    		echo "shutdown $option" | ${ORACLE_HOME}/bin/sqlplus '/ as sysdba';
  fi
EOF1

n=0
for I in ${LISTENER_NAME[@]}
do
  $CMD ${ORACLE_ADMIN} -c "/bin/bash" <<EOF1
  export ORACLE_HOME=${ORACLE_HOME}

  if [[ "$INSTANCE_TYPE" = "database" ]] && [[ "$LISTENER" = "yes" ]]; then
      password=""
      if [[ ! -z "${LISTENER_PASS[${n}]}" ]]
      then
          password="set password ${LISTENER_PASS[${n}]}"
      fi

      ${ORACLE_HOME}/bin/lsnrctl <<LSN
      \${password}
      status ${I}
      exit
LSN
  fi
EOF1

varstatus=$?
if [[ $varstatus -eq 0 ]] && [[ "$LISTENER" = "yes" ]]; then
     if [[ "$INSTANCE_TYPE" = "database" ]]; then
          $CMD ${ORACLE_ADMIN} -c "/bin/bash" << EOF2

          export ORACLE_HOME=${ORACLE_HOME}
          export ORACLE_SID=${SID_NAME}

          password=""
          if [[ ! -z "${LISTENER_PASS[${n}]}" ]]
          then
               password="set password ${LISTENER_PASS[${n}]}"
          fi
          ${ORACLE_HOME}/bin/lsnrctl <<LSN
          \${password}
          stop ${I}
          exit
LSN
EOF2
     fi
fi
(( n = n + 1 ))
done

oracle_stop=$?
return $oracle_stop

}

###############################################################################
# Start Main
###############################################################################

SLEEP=/bin/sleep
PGREP=/usr/bin/pgrep
AWK=/bin/awk
GREP=/bin/grep
OCSSD="$ASM_HOME/bin/ocssd.bin"

########################################
# source the haoracle_utils.sh file
#######################################
. $TKIT_SCRIPT_DIR/haoracle_utils.sh


if [[ $# -lt 1 ]]; then
    echo "Usage: ${0} [ start | abort | immediate | cleanup ]"
    exit 1
fi

#determine_db_type function requires oracle to be running
#hence if the argument is other than start means, it can be
#cleanup, abort or immediate. During that time Oracle will
#be avaiable.
if [[ "$1" != "start" && "$START_MODE" = "open" ]]; then
   #Check if the DB instance is a CDB, if yes then stop the PDB associated
   #with it if the START_MODE of the CDB is "open"
   CDB=$(determine_db_type)
fi

case $1 in

start)
    start_oracle
    check_status=$?
    return $check_status

;;

cleanup)
    cleanup_oracle
;;

abort)
    shutdown_oracle abort
    check_status=$?
    return $check_status
;;

immediate)
    shutdown_oracle immediate
    check_status=$?
    return $check_status
;;

*)
    echo "Usage: ${0} [ start | abort | immediate | cleanup ]"
;;
esac
